# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Microsoft Terminology Collection."""


import os
import xml.etree.ElementTree as ElementTree

import datasets


_DESCRIPTION = """\
The Microsoft Terminology Collection can be used to develop localized versions of applications that integrate with Microsoft products.
It can also be used to integrate Microsoft terminology into other terminology collections or serve as a base IT glossary
for language development in the nearly 100 languages available. Terminology is provided in .tbx format, an industry standard for terminology exchange.
"""

_LICENSE = """\
See the Microsoft Language Portal Materials License and the Microsoft Terms of Use for details.
"""

_ENTRY_ID = "entry_id"
_TERM_SOURCE = "term_source"
_TERM_POS = "pos"
_TERM_DEFINITION = "definition"
_TERM_TARGET = "term_target"

_FILENAME = "MicrosoftTermCollection.tbx"


class MsTerms(datasets.GeneratorBasedBuilder):
    """The Microsoft Terminology Collection."""

    VERSION = datasets.Version("1.0.0")

    @property
    def manual_download_instructions(self):
        return """\
    You need to go to https://www.microsoft.com/en-us/language/terminology,
    and manually download the language of your interest. Once it is completed,
    a file named MicrosoftTermCollection.tbx will be appeared in your Downloads folder
    or whichever folder your browser chooses to save files to.
    You can then move MicrosoftTermCollection.tbx under <path/to/folder>.
    The <path/to/folder> can e.g. be "~/manual_data".
    ms_terms can then be loaded using the following command `datasets.load_dataset("ms_terms", data_dir="<path/to/folder>")`.
    """

    def _info(self):
        feature_names = [_ENTRY_ID, _TERM_SOURCE, _TERM_POS, _TERM_DEFINITION, _TERM_TARGET]
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features({k: datasets.Value("string") for k in feature_names}),
            supervised_keys=None,
            homepage="https://www.microsoft.com/en-us/language/terminology",
            citation="",
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        path_to_manual_file = os.path.join(os.path.abspath(os.path.expanduser(dl_manager.manual_dir)), _FILENAME)

        if not os.path.exists(path_to_manual_file):
            raise FileNotFoundError(
                f"{path_to_manual_file} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('ms_terms', data_dir=...)` that includes a file name {_FILENAME}. Manual download instructions: {self.manual_download_instructions})"
            )
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"path": path_to_manual_file})]

    def _generate_examples(self, path=None, title_set=None):
        tree = ElementTree.parse(path)
        root = tree.getroot()
        for i, entry in enumerate(root.findall(".//termEntry")):
            entry_id = entry.attrib.get("id")
            langsets = entry.findall("./langSet")
            if len(langsets) != 2:
                continue

            term_source = langsets[0].find(".//term").text
            term_definition = langsets[0].find(".//descrip").text
            term_pos = langsets[0].find(".//termNote").text
            term_target = langsets[1].find(".//term").text
            yield i, {
                _ENTRY_ID: entry_id,
                _TERM_SOURCE: term_source,
                _TERM_POS: term_pos,
                _TERM_DEFINITION: term_definition,
                _TERM_TARGET: term_target,
            }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The eHealth-KD 2020 Corpus."""


import datasets


_CITATION = """\
@inproceedings{overview_ehealthkd2020,
  author    = {Piad{-}Morffis, Alejandro and
               Guti{\'{e}}rrez, Yoan and
               Cañizares-Diaz, Hian and
               Estevez{-}Velarde, Suilan and
               Almeida{-}Cruz, Yudivi{\'{a}}n and
               Muñoz, Rafael and
               Montoyo, Andr{\'{e}}s},
  title     = {Overview of the eHealth Knowledge Discovery Challenge at IberLEF 2020},
  booktitle = ,
  year      = {2020},
}
"""

_DESCRIPTION = """\
Dataset of the eHealth Knowledge Discovery Challenge at IberLEF 2020. It is designed for
the identification of semantic entities and relations in Spanish health documents.
"""

_HOMEPAGE = "https://knowledge-learning.github.io/ehealthkd-2020/"

_LICENSE = "https://creativecommons.org/licenses/by-nc-sa/4.0/"

_URL = "https://raw.githubusercontent.com/knowledge-learning/ehealthkd-2020/master/data/"
_TRAIN_DIR = "training/"
_DEV_DIR = "development/main/"
_TEST_DIR = "testing/scenario3-taskB/"
_TEXT_FILE = "scenario.txt"
_ANNOTATIONS_FILE = "scenario.ann"


class EhealthKD(datasets.GeneratorBasedBuilder):
    """The eHealth-KD 2020 Corpus."""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="ehealth_kd", version=VERSION, description="eHealth-KD Corpus"),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "entities": [
                        {
                            "ent_id": datasets.Value("string"),
                            "ent_text": datasets.Value("string"),
                            "ent_label": datasets.ClassLabel(names=["Concept", "Action", "Predicate", "Reference"]),
                            "start_character": datasets.Value("int32"),
                            "end_character": datasets.Value("int32"),
                        }
                    ],
                    "relations": [
                        {
                            "rel_id": datasets.Value("string"),
                            "rel_label": datasets.ClassLabel(
                                names=[
                                    "is-a",
                                    "same-as",
                                    "has-property",
                                    "part-of",
                                    "causes",
                                    "entails",
                                    "in-time",
                                    "in-place",
                                    "in-context",
                                    "subject",
                                    "target",
                                    "domain",
                                    "arg",
                                ]
                            ),
                            "arg1": datasets.Value("string"),
                            "arg2": datasets.Value("string"),
                        }
                    ],
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        urls_to_download = {
            k: [f"{_URL}{v}{_TEXT_FILE}", f"{_URL}{v}{_ANNOTATIONS_FILE}"]
            for k, v in zip(["train", "dev", "test"], [_TRAIN_DIR, _DEV_DIR, _TEST_DIR])
        }

        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"txt_path": downloaded_files["train"][0], "ann_path": downloaded_files["train"][1]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"txt_path": downloaded_files["dev"][0], "ann_path": downloaded_files["dev"][1]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"txt_path": downloaded_files["test"][0], "ann_path": downloaded_files["test"][1]},
            ),
        ]

    def _generate_examples(self, txt_path, ann_path):
        """Yields examples."""
        with open(txt_path, encoding="utf-8") as txt_file, open(ann_path, encoding="utf-8") as ann_file:
            _id = 0
            entities = []
            relations = []

            annotations = ann_file.readlines()
            last = annotations[-1]

            # Create a variable to keep track of the last annotation (entity or relation) to know when a sentence is fully annotated
            # In the annotations file, the entities are before the relations
            last_annotation = ""

            for annotation in annotations:
                if annotation == last:
                    sentence = txt_file.readline().strip()
                    yield _id, {"sentence": sentence, "entities": entities, "relations": relations}

                if annotation.startswith("T"):
                    if last_annotation == "relation":
                        sentence = txt_file.readline().strip()
                        yield _id, {"sentence": sentence, "entities": entities, "relations": relations}
                        _id += 1
                        entities = []
                        relations = []

                    ent_id, mid, ent_text = annotation.strip().split("\t")
                    ent_label, spans = mid.split(" ", 1)
                    start_character = spans.split(" ")[0]
                    end_character = spans.split(" ")[-1]

                    entities.append(
                        {
                            "ent_id": ent_id,
                            "ent_text": ent_text,
                            "ent_label": ent_label,
                            "start_character": start_character,
                            "end_character": end_character,
                        }
                    )

                    last_annotation = "entity"

                else:
                    rel_id, rel_label, arg1, arg2 = annotation.strip().split()
                    if annotation.startswith("R"):
                        arg1 = arg1.split(":")[1]
                        arg2 = arg2.split(":")[1]

                    relations.append({"rel_id": rel_id, "rel_label": rel_label, "arg1": arg1, "arg2": arg2})

                    last_annotation = "relation"



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Quora question pairs data"""


import csv

import datasets


_DESCRIPTION = "The Quora dataset is composed of question pairs, and the task is to determine if the questions are paraphrases of each other (have the same meaning)."

_URL = "http://qim.fs.quoracdn.net/quora_duplicate_questions.tsv"


class Quora(datasets.GeneratorBasedBuilder):
    """Quora Question Pairs dataset"""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "questions": datasets.features.Sequence(
                        {
                            "id": datasets.Value("int32"),
                            "text": datasets.Value("string"),
                        }
                    ),
                    "is_duplicate": datasets.Value("bool"),
                }
            ),
            homepage="https://www.quora.com/q/quoradata/First-Quora-Dataset-Release-Question-Pairs",
        )

    def _split_generators(self, dl_manager):
        data_file = dl_manager.download_and_extract({"data_file": _URL})
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs=data_file)]

    def _generate_examples(self, data_file):
        with open(data_file, encoding="utf-8") as f:
            data = csv.DictReader(f, delimiter="\t")
            for idx, row in enumerate(data):
                yield idx, {
                    "questions": [
                        {"id": row["qid1"], "text": row["question1"]},
                        {"id": row["qid2"], "text": row["question2"]},
                    ],
                    "is_duplicate": row["is_duplicate"] == "1",
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Spanish Billion Words Corpus."""


import os

import datasets


_CITATION = """\
@misc{cardellinoSBWCE,
     author = {Cardellino, Cristian},
     title = {Spanish {B}illion {W}ords {C}orpus and {E}mbeddings},
     url = {https://crscardellino.github.io/SBWCE/},
     month = {August},
     year = {2019}
}
"""

_DESCRIPTION = """\
An unannotated Spanish corpus of nearly 1.5 billion words, compiled from different resources from the web.
This resources include the spanish portions of SenSem, the Ancora Corpus, some OPUS Project Corpora and the Europarl,
the Tibidabo Treebank, the IULA Spanish LSP Treebank, and dumps from the Spanish Wikipedia, Wikisource and Wikibooks.
This corpus is a compilation of 100 text files. Each line of these files represents one of the 50 million sentences from the corpus.
"""

_HOMEPAGE = "https://crscardellino.github.io/SBWCE/"

_LICENSE = "https://creativecommons.org/licenses/by-sa/4.0/"

_URL = "http://cs.famaf.unc.edu.ar/~ccardellino/SBWCE/clean_corpus.tar.bz2"


class SpanishBillionWords(datasets.GeneratorBasedBuilder):
    """The Spanish Billion Words Corpus."""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="corpus",
            version=VERSION,
            description="100 text files where each line represents a sentence from the corpus",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"directory": os.path.join(data_dir, "spanish_billion_words")}
            )
        ]

    def _generate_examples(self, directory):
        """Yields examples."""
        files = os.listdir(directory)
        files = sorted(files)
        _id = 0

        for file in files:
            file_path = os.path.join(directory, file)
            with open(file_path, mode="r", encoding="utf-8") as f:
                for line in f:
                    yield _id, {"text": line.strip()}
                    _id += 1



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Neural-Code-Search-Evaluation-Dataset presents an evaluation dataset consisting of natural language query and code snippet pairs"""


import json
from itertools import chain

import datasets


_CITATION = """\
@InProceedings{huggingface:dataset,
title         = {Neural Code Search Evaluation Dataset},
authors       = {Hongyu Li, Seohyun Kim and Satish Chandra},
journal       = {arXiv e-prints},
year          = 2018,
eid           = {arXiv:1908.09804 [cs.SE]},
pages         = {arXiv:1908.09804 [cs.SE]},
archivePrefix = {arXiv},
eprint        = {1908.09804},
}
"""

_DESCRIPTION = """\
Neural-Code-Search-Evaluation-Dataset presents an evaluation dataset \
consisting of natural language query and code snippet pairs and a search corpus \
consisting of code snippets collected from the most popular Android repositories \
on GitHub.
"""

_HOMEPAGE = "https://github.com/facebookresearch/Neural-Code-Search-Evaluation-Dataset/tree/master/data"

_LICENSE = "CC-BY-NC 4.0 (Attr Non-Commercial Inter.)"

_BASE_URL = "https://raw.githubusercontent.com/facebookresearch/Neural-Code-Search-Evaluation-Dataset/master/data/"
_URLs = {
    "evaluation_dataset": _BASE_URL + "287_android_questions.json",
    "search_corpus_1": _BASE_URL + "search_corpus_1.tar.gz",
    "search_corpus_2": _BASE_URL + "search_corpus_2.tar.gz",
}


class NeuralCodeSearch(datasets.GeneratorBasedBuilder):
    """Neural Code Search Evaluation Dataset"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="evaluation_dataset",
            version=VERSION,
            description="The evaluation dataset is composed of \
            287 Stack Overflow question and answer pairs",
        ),
        datasets.BuilderConfig(
            name="search_corpus",
            version=VERSION,
            description="The search corpus is indexed using all \
            method bodies parsed from the 24,549 GitHub repositories.",
        ),
    ]

    FILENAME_MAP = {
        "evaluation_dataset": "287_android_questions.json",
        "search_corpus": "search_corpus_1.jsonl",
    }

    def _info(self):
        if self.config.name == "evaluation_dataset":
            features = datasets.Features(
                {
                    "stackoverflow_id": datasets.Value("int32"),
                    "question": datasets.Value("string"),
                    "question_url": datasets.Value("string"),
                    "question_author": datasets.Value("string"),
                    "question_author_url": datasets.Value("string"),
                    "answer": datasets.Value("string"),
                    "answer_url": datasets.Value("string"),
                    "answer_author": datasets.Value("string"),
                    "answer_author_url": datasets.Value("string"),
                    "examples": datasets.features.Sequence(datasets.Value("int32")),
                    "examples_url": datasets.features.Sequence(datasets.Value("string")),
                }
            )
        else:
            features = datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "filepath": datasets.Value("string"),
                    "method_name": datasets.Value("string"),
                    "start_line": datasets.Value("int32"),
                    "end_line": datasets.Value("int32"),
                    "url": datasets.Value("string"),
                }
            )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        if self.config.name == "evaluation_dataset":
            filepath = dl_manager.download_and_extract(_URLs[self.config.name])
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={"filepath": filepath},
                ),
            ]
        else:
            my_urls = [url for config, url in _URLs.items() if config.startswith(self.config.name)]
            archives = dl_manager.download(my_urls)
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "files": chain(*(dl_manager.iter_archive(archive) for archive in archives)),
                    },
                ),
            ]

    def _generate_examples(self, filepath=None, files=None):
        """Yields examples."""
        id_ = 0
        if self.config.name == "evaluation_dataset":
            with open(filepath, encoding="utf-8") as f:
                data = json.load(f)
                for row in data:
                    yield id_, {
                        "stackoverflow_id": row["stackoverflow_id"],
                        "question": row["question"],
                        "question_url": row["question_url"],
                        "question_author": row["question_author"],
                        "question_author_url": row["question_author_url"],
                        "answer": row["answer"],
                        "answer_url": row["answer_url"],
                        "answer_author": row["answer_author"],
                        "answer_author_url": row["answer_author_url"],
                        "examples": row["examples"],
                        "examples_url": row["examples_url"],
                    }
                    id_ += 1
        else:
            for _, f in files:
                for row in f:
                    data_dict = json.loads(row.decode("utf-8"))
                    yield id_, {
                        "id": data_dict["id"],
                        "filepath": data_dict["filepath"],
                        "method_name": data_dict["method_name"],
                        "start_line": data_dict["start_line"],
                        "end_line": data_dict["end_line"],
                        "url": data_dict["url"],
                    }
                    id_ += 1



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""HEAD-QA: A Healthcare Dataset for Complex Reasoning"""


import json
import os

import datasets


_CITATION = """\
@inproceedings{vilares-gomez-rodriguez-2019-head,
    title = "{HEAD}-{QA}: A Healthcare Dataset for Complex Reasoning",
    author = "Vilares, David  and
      G{\'o}mez-Rodr{\'i}guez, Carlos",
    booktitle = "Proceedings of the 57th Annual Meeting of the Association for Computational Linguistics",
    month = jul,
    year = "2019",
    address = "Florence, Italy",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/P19-1092",
    doi = "10.18653/v1/P19-1092",
    pages = "960--966",
    abstract = "We present HEAD-QA, a multi-choice question answering testbed to encourage research on complex reasoning. The questions come from exams to access a specialized position in the Spanish healthcare system, and are challenging even for highly specialized humans. We then consider monolingual (Spanish) and cross-lingual (to English) experiments with information retrieval and neural techniques. We show that: (i) HEAD-QA challenges current methods, and (ii) the results lag well behind human performance, demonstrating its usefulness as a benchmark for future work.",
}
"""

_DESCRIPTION = """\
HEAD-QA is a multi-choice HEAlthcare Dataset. The questions come from exams to access a specialized position in the
Spanish healthcare system, and are challenging even for highly specialized humans. They are designed by the Ministerio
de Sanidad, Consumo y Bienestar Social.

The dataset contains questions about the following topics: medicine, nursing, psychology, chemistry, pharmacology and biology.
"""

_HOMEPAGE = "https://aghie.github.io/head-qa/"

_LICENSE = "MIT License"

_URL = "https://drive.google.com/u/0/uc?export=download&id=1a_95N5zQQoUCq8IBNVZgziHbeM-QxG2t"

_DIRS = {"es": "HEAD", "en": "HEAD_EN"}


class HeadQA(datasets.GeneratorBasedBuilder):
    """HEAD-QA: A Healthcare Dataset for Complex Reasoning"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="es", version=VERSION, description="Spanish HEAD dataset"),
        datasets.BuilderConfig(name="en", version=VERSION, description="English HEAD dataset"),
    ]

    DEFAULT_CONFIG_NAME = "es"

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "name": datasets.Value("string"),
                    "year": datasets.Value("string"),
                    "category": datasets.Value("string"),
                    "qid": datasets.Value("int32"),
                    "qtext": datasets.Value("string"),
                    "ra": datasets.Value("int32"),
                    "image": datasets.Image(),
                    "answers": [
                        {
                            "aid": datasets.Value("int32"),
                            "atext": datasets.Value("string"),
                        }
                    ],
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URL)

        dir = _DIRS[self.config.name]
        data_lang_dir = os.path.join(data_dir, dir)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"data_dir": data_dir, "filepath": os.path.join(data_lang_dir, f"train_{dir}.json")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"data_dir": data_dir, "filepath": os.path.join(data_lang_dir, f"test_{dir}.json")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"data_dir": data_dir, "filepath": os.path.join(data_lang_dir, f"dev_{dir}.json")},
            ),
        ]

    def _generate_examples(self, data_dir, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            head_qa = json.load(f)
            for exam_id, exam in enumerate(head_qa["exams"]):
                content = head_qa["exams"][exam]
                name = content["name"].strip()
                year = content["year"].strip()
                category = content["category"].strip()
                for question in content["data"]:
                    qid = int(question["qid"].strip())
                    qtext = question["qtext"].strip()
                    ra = int(question["ra"].strip())
                    image_path = question["image"].strip()

                    aids = [answer["aid"] for answer in question["answers"]]
                    atexts = [answer["atext"].strip() for answer in question["answers"]]
                    answers = [{"aid": aid, "atext": atext} for aid, atext in zip(aids, atexts)]

                    id_ = f"{exam_id}_{qid}"
                    yield id_, {
                        "name": name,
                        "year": year,
                        "category": category,
                        "qid": qid,
                        "qtext": qtext,
                        "ra": ra,
                        "image": os.path.join(data_dir, image_path) if image_path else None,
                        "answers": answers,
                    }



"""TODO(math_qa): Add a description here."""


import json
import os

import datasets


# TODO(math_qa): BibTeX citation
_CITATION = """
"""

# TODO(math_qa):
_DESCRIPTION = """
Our dataset is gathered by using a new representation language to annotate over the AQuA-RAT dataset. AQuA-RAT has provided the questions, options, rationale, and the correct options.
"""
_URL = "https://math-qa.github.io/math-QA/data/MathQA.zip"


class MathQa(datasets.GeneratorBasedBuilder):
    """TODO(math_qa): Short description of my dataset."""

    # TODO(math_qa): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(math_qa): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    # These are the features of your dataset like images, labels ...
                    "Problem": datasets.Value("string"),
                    "Rationale": datasets.Value("string"),
                    "options": datasets.Value("string"),
                    "correct": datasets.Value("string"),
                    "annotated_formula": datasets.Value("string"),
                    "linear_formula": datasets.Value("string"),
                    "category": datasets.Value("string"),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://math-qa.github.io/math-QA/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(math_qa): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_path = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_path, "train.json")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_path, "test.json")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_path, "dev.json")},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(math_qa): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for id_, row in enumerate(data):
                yield id_, row



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""OrangeSum dataset"""


import datasets


_CITATION = """\
@article{eddine2020barthez,
  title={BARThez: a Skilled Pretrained French Sequence-to-Sequence Model},
  author={Eddine, Moussa Kamal and Tixier, Antoine J-P and Vazirgiannis, Michalis},
  journal={arXiv preprint arXiv:2010.12321},
  year={2020}
}
"""

_DESCRIPTION = """\
The OrangeSum dataset was inspired by the XSum dataset. It was created by scraping the "Orange Actu" website: https://actu.orange.fr/. Orange S.A. is a large French multinational telecommunications corporation, with 266M customers worldwide. Scraped pages cover almost a decade from Feb 2011 to Sep 2020. They belong to five main categories: France, world, politics, automotive, and society. The society category is itself divided into 8 subcategories: health, environment, people, culture, media, high-tech, unsual ("insolite" in French), and miscellaneous.

Each article featured a single-sentence title as well as a very brief abstract, both professionally written by the author of the article. These two fields were extracted from each page, thus creating two summarization tasks: OrangeSum Title and OrangeSum Abstract.
"""

_URL_DATA = {
    "abstract": "https://raw.githubusercontent.com/Tixierae/OrangeSum/main/data/docs/splits/abstract.tgz",
    "title": "https://raw.githubusercontent.com/Tixierae/OrangeSum/main/data/docs/splits/title.tgz",
}

_DOCUMENT = "text"
_SUMMARY = "summary"


class OrangeSum(datasets.GeneratorBasedBuilder):
    """OrangeSum: a french abstractive summarization dataset"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="abstract", description="Abstracts used as summaries", version=VERSION),
        datasets.BuilderConfig(name="title", description="Titles used as summaries", version=VERSION),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    _DOCUMENT: datasets.Value("string"),
                    _SUMMARY: datasets.Value("string"),
                }
            ),
            supervised_keys=(_DOCUMENT, _SUMMARY),
            homepage="https://github.com/Tixierae/OrangeSum/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        archive = dl_manager.download(_URL_DATA[self.config.name])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "source_files": dl_manager.iter_archive(archive),
                    "target_files": dl_manager.iter_archive(archive),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "source_files": dl_manager.iter_archive(archive),
                    "target_files": dl_manager.iter_archive(archive),
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "source_files": dl_manager.iter_archive(archive),
                    "target_files": dl_manager.iter_archive(archive),
                    "split": "valid",
                },
            ),
        ]

    def _generate_examples(self, source_files, target_files, split):
        """Yields examples."""
        expected_source_path = f"{self.config.name}/{split}.source"
        expected_target_path = f"{self.config.name}/{split}.target"
        for source_path, f_source in source_files:
            if source_path == expected_source_path:
                for target_path, f_target in target_files:
                    if target_path == expected_target_path:
                        for idx, (document, summary) in enumerate(zip(f_source, f_target)):
                            yield idx, {_DOCUMENT: document.decode("utf-8"), _SUMMARY: summary.decode("utf-8")}
                        break
                break



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""ID Puisi: A Dataset for Indonesian Poem"""


import csv

import datasets


_DESCRIPTION = """\
Puisi (poem) is an Indonesian poetic form. The dataset contains 7223 Indonesian puisi with its title and author.
"""

_TRAIN_DOWNLOAD_URL = "https://raw.githubusercontent.com/ilhamfp/puisi-pantun-generator/main/data/puisi.csv"


class IDPuisi(datasets.GeneratorBasedBuilder):
    """ID Puisi: A Datases for Indonesian Poem"""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "title": datasets.Value("string"),
                    "author": datasets.Value("string"),
                    "puisi": datasets.Value("string"),
                    "puisi_with_header": datasets.Value("string"),
                }
            ),
            homepage="https://github.com/ilhamfp/puisi-pantun-generator",
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_TRAIN_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                },
            ),
        ]

    def _generate_examples(self, filepath):
        """Generate ID Puisi examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(
                csv_file,
                delimiter=",",
                quoting=csv.QUOTE_ALL,
            )
            for id_, row in enumerate(csv_reader):
                if id_ == 0:
                    # Skip header
                    continue

                yield id_, {"puisi": row[0], "title": row[1], "author": row[2], "puisi_with_header": row[3]}



"""C4 dataset based on Common Crawl."""


import gzip
import json

import datasets


logger = datasets.logging.get_logger(__name__)


_DESCRIPTION = """\
A colossal, cleaned version of Common Crawl's web crawl corpus.

Based on Common Crawl dataset: "https://commoncrawl.org".

This is the processed version of Google's C4 dataset by AllenAI.
"""

_CITATION = """
@article{2019t5,
    author = {Colin Raffel and Noam Shazeer and Adam Roberts and Katherine Lee and Sharan Narang and Michael Matena and Yanqi Zhou and Wei Li and Peter J. Liu},
    title = {Exploring the Limits of Transfer Learning with a Unified Text-to-Text Transformer},
    journal = {arXiv e-prints},
    year = {2019},
    archivePrefix = {arXiv},
    eprint = {1910.10683},
}
"""

_URL = "https://github.com/allenai/allennlp/discussions/5056"

_VARIANTS = ["en", "realnewslike", "en.noblocklist", "en.noclean"]

_N_SHARDS_PER_SPLIT = {
    "en": {"train": 1024, "validation": 8},
    "realnewslike": {"train": 512, "validation": 1},
    "en.noblocklist": {"train": 1024, "validation": 8},
    "en.noclean": {"train": 7168, "validation": 64},
}

_DATA_URL = "https://huggingface.co/datasets/allenai/c4/resolve/1ddc917116b730e1859edef32896ec5c16be51d0/{name}/c4-{split}.{index:05d}-of-{n_shards:05d}.json.gz"


class C4(datasets.GeneratorBasedBuilder):
    """C4, a colossal, cleaned version of Common Crawl's web crawl corpus."""

    BUILDER_CONFIGS = [datasets.BuilderConfig(name) for name in _VARIANTS]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "timestamp": datasets.Value("string"),
                    "url": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage=_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        data_urls = {}
        for split in ["train", "validation"]:
            n_shards = _N_SHARDS_PER_SPLIT[self.config.name][split]
            data_urls[split] = [
                _DATA_URL.format(name=self.config.name, split=split, index=index, n_shards=n_shards)
                for index in range(n_shards)
            ]
        train_downloaded_files = dl_manager.download(data_urls["train"])
        validation_downloaded_files = dl_manager.download(data_urls["validation"])
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepaths": train_downloaded_files}),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION, gen_kwargs={"filepaths": validation_downloaded_files}
            ),
        ]

    def _generate_examples(self, filepaths):
        """This function returns the examples in the raw (text) form by iterating on all the files."""
        id_ = 0
        for filepath in filepaths:
            logger.info("generating examples from = %s", filepath)
            with gzip.open(open(filepath, "rb"), "rt", encoding="utf-8") as f:
                for line in f:
                    if line:
                        example = json.loads(line)
                        yield id_, example
                        id_ += 1



"""TODO(sciQ): Add a description here."""


import json
import os

import datasets


# TODO(sciQ): BibTeX citation
_CITATION = """\
@inproceedings{SciQ,
    title={Crowdsourcing Multiple Choice Science Questions},
    author={Johannes Welbl, Nelson F. Liu, Matt Gardner},
    year={2017},
    journal={arXiv:1707.06209v1}
}
"""

# TODO(sciQ):
_DESCRIPTION = """\
The SciQ dataset contains 13,679 crowdsourced science exam questions about Physics, Chemistry and Biology, among others. The questions are in multiple-choice format with 4 answer options each. For the majority of the questions, an additional paragraph with supporting evidence for the correct answer is provided.

"""
_URL = "https://s3-us-west-2.amazonaws.com/ai2-website/data/SciQ.zip"


class Sciq(datasets.GeneratorBasedBuilder):
    """TODO(sciQ): Short description of my dataset."""

    # TODO(sciQ): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(sciQ): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    # These are the features of your dataset like images, labels ...
                    "question": datasets.Value("string"),
                    "distractor3": datasets.Value("string"),
                    "distractor1": datasets.Value("string"),
                    "distractor2": datasets.Value("string"),
                    "correct_answer": datasets.Value("string"),
                    "support": datasets.Value("string"),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://allenai.org/data/sciq",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(sciQ): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        data_dir = os.path.join(dl_dir, "SciQ dataset-2 3")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(data_dir, "train.json")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(data_dir, "valid.json")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(data_dir, "test.json")},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(sciQ): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for id_, row in enumerate(data):
                yield id_, row



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Authorship Analysis: Style Change Detection"""


import json
import os

import datasets


_CITATION = """\
@inproceedings{bevendorff2020shared,
  title={Shared Tasks on Authorship Analysis at PAN 2020},
  author={Bevendorff, Janek and Ghanem, Bilal and Giachanou, Anastasia and Kestemont, Mike and Manjavacas, Enrique and Potthast, Martin and Rangel, Francisco and Rosso, Paolo and Specht, G{\"u}nther and Stamatatos, Efstathios and others},
  booktitle={European Conference on Information Retrieval},
  pages={508--516},
  year={2020},
  organization={Springer}
}
"""

_DESCRIPTION = """\
The goal of the style change detection task is to identify text positions within a given multi-author document at which the author switches. Detecting these positions is a crucial part of the authorship identification process, and for multi-author document analysis in general.

Access to the dataset needs to be requested from zenodo.
"""


class StyleChangeDetection(datasets.GeneratorBasedBuilder):
    """Style Change Detection Dataset from PAN20"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="narrow",
            version=datasets.Version("1.0.0", "Version 1"),
            description="The narrow subset contains texts from a relatively narrow set of subjects matters (all related to technology).",
        ),
        datasets.BuilderConfig(
            name="wide",
            version=datasets.Version("1.0.0", "Version 1"),
            description="The wide subset adds additional subject areas (travel, philosophy, economics, history, etc.).",
        ),
    ]

    @property
    def manual_download_instructions(self):
        return """\
  You should download the dataset from https://zenodo.org/record/3660984
  The dataset needs requesting.

  Download each file, extract it and place in a dir of your choice,
  which will be used as a manual_dir, e.g. `~/.manual_dirs/style_change_detection`
  Style Change Detection can then be loaded via:
  `datasets.load_dataset("style_change_detection", data_dir="~/.manual_dirs/style_change_detection")`.
  """

    def _info(self):
        features = {
            "id": datasets.Value("string"),
            "text": datasets.Value("string"),
            "authors": datasets.Value("int32"),
            "structure": datasets.features.Sequence(datasets.Value("string")),
            "site": datasets.Value("string"),
            "multi-author": datasets.Value("bool"),
            "changes": datasets.features.Sequence(datasets.Value("bool")),
        }

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(features),
            homepage="https://pan.webis.de/clef20/pan20-web/style-change-detection.html",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))

        train_dir = os.path.join(data_dir, "train", "dataset-" + self.config.name)
        val_dir = os.path.join(data_dir, "validation", "dataset-" + self.config.name)

        if not os.path.exists(train_dir):
            raise FileNotFoundError(
                f"{train_dir} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('style_change_detection', data_dir=...)` that includes {train_dir}. Manual download instructions: {self.manual_download_instructions}"
            )

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "articles": [f for f in os.listdir(train_dir) if f.endswith(".txt")],
                    "base_dir": train_dir,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"articles": [f for f in os.listdir(val_dir) if f.endswith(".txt")], "base_dir": val_dir},
            ),
        ]

    def _generate_examples(self, articles=None, base_dir=None):
        """Yields examples."""
        for idx, article_filename in enumerate(articles):
            label_path = os.path.join(base_dir, "truth-" + article_filename[:-4] + ".json")
            with open(label_path, encoding="utf-8") as f:
                example = json.load(f)
                example["id"] = article_filename[8:-4]
                example["text"] = open(os.path.join(base_dir, article_filename), encoding="utf-8").read()

                # Convert integers into boolean
                example["multi-author"] = example["multi-author"] == 1
                for i in range(len(example["changes"])):
                    example["changes"][i] = example["changes"][i] == 1

                yield idx, example



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Facebook Low Resource (FLoRes) machine translation benchmark dataset."""


import collections

import datasets


_DESCRIPTION = """\
Evaluation datasets for low-resource machine translation: Nepali-English and Sinhala-English.
"""

_CITATION = """\
@misc{guzmn2019new,
    title={Two New Evaluation Datasets for Low-Resource Machine Translation: Nepali-English and Sinhala-English},
    author={Francisco Guzman and Peng-Jen Chen and Myle Ott and Juan Pino and Guillaume Lample and Philipp Koehn and Vishrav Chaudhary and Marc'Aurelio Ranzato},
    year={2019},
    eprint={1902.01382},
    archivePrefix={arXiv},
    primaryClass={cs.CL}
}
"""

_DATA_URL = "https://github.com/facebookresearch/flores/raw/main/floresv1/data/wikipedia_en_ne_si_test_sets.tgz"

# Tuple that describes a single pair of files with matching translations.
# language_to_file is the map from language (2 letter string: example 'en')
# to the file path in the extracted directory.
TranslateData = collections.namedtuple("TranslateData", ["url", "language_to_file"])


class FloresConfig(datasets.BuilderConfig):
    """BuilderConfig for FLoRes."""

    def __init__(self, language_pair=(None, None), **kwargs):
        """BuilderConfig for FLoRes.

        Args:
            for the `datasets.features.text.TextEncoder` used for the features feature.
          language_pair: pair of languages that will be used for translation. Should
            contain 2-letter coded strings. First will be used at source and second
            as target in supervised mode. For example: ("se", "en").
          **kwargs: keyword arguments forwarded to super.
        """
        name = "%s%s" % (language_pair[0], language_pair[1])

        description = ("Translation dataset from %s to %s") % (language_pair[0], language_pair[1])
        super(FloresConfig, self).__init__(
            name=name,
            description=description,
            version=datasets.Version("1.1.0", ""),
            **kwargs,
        )

        # Validate language pair.
        assert "en" in language_pair, ("Config language pair must contain `en`, got: %s", language_pair)
        source, target = language_pair
        non_en = source if target == "en" else target
        assert non_en in ["ne", "si"], ("Invalid non-en language in pair: %s", non_en)

        self.language_pair = language_pair


class Flores(datasets.GeneratorBasedBuilder):
    """FLoRes machine translation dataset."""

    BUILDER_CONFIGS = [
        FloresConfig(
            language_pair=("ne", "en"),
        ),
        FloresConfig(
            language_pair=("si", "en"),
        ),
    ]

    def _info(self):
        source, target = self.config.language_pair
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {"translation": datasets.features.Translation(languages=self.config.language_pair)}
            ),
            supervised_keys=(source, target),
            homepage="https://github.com/facebookresearch/flores/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        archive = dl_manager.download(_DATA_URL)

        source, target = self.config.language_pair
        non_en = source if target == "en" else target
        path_tmpl = "wikipedia_en_ne_si_test_sets/wikipedia.{split}.{non_en}-en." "{lang}"

        files = {}
        for split in ("dev", "devtest"):
            files[split] = {
                "source_file": path_tmpl.format(split=split, non_en=non_en, lang=source),
                "target_file": path_tmpl.format(split=split, non_en=non_en, lang=target),
                "files": dl_manager.iter_archive(archive),
            }

        return [
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs=files["dev"]),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs=files["devtest"]),
        ]

    def _generate_examples(self, files, source_file, target_file):
        """This function returns the examples in the raw (text) form."""
        source_sentences, target_sentences = None, None
        for path, f in files:
            if path == source_file:
                source_sentences = f.read().decode("utf-8").split("\n")
            elif path == target_file:
                target_sentences = f.read().decode("utf-8").split("\n")
            if source_sentences is not None and target_sentences is not None:
                break

        assert len(target_sentences) == len(source_sentences), "Sizes do not match: %d vs %d for %s vs %s." % (
            len(source_sentences),
            len(target_sentences),
            source_file,
            target_file,
        )

        source, target = self.config.language_pair
        for idx, (l1, l2) in enumerate(zip(source_sentences, target_sentences)):
            result = {"translation": {source: l1, target: l2}}
            # Make sure that both translations are non-empty.
            if all(result.values()):
                yield idx, result



"""TODO(commonsense_qa): Add a description here."""


import json

import datasets


# TODO(commonsense_qa): BibTeX citation
_CITATION = """\
@InProceedings{commonsense_QA,
title={COMMONSENSEQA: A Question Answering Challenge Targeting Commonsense Knowledge},
author={Alon, Talmor and Jonathan, Herzig and Nicholas, Lourie and Jonathan ,Berant},
journal={arXiv preprint arXiv:1811.00937v2},
year={2019}

"""

# TODO(commonsense_qa):
_DESCRIPTION = """\
CommonsenseQA is a new multiple-choice question answering dataset that requires different types of commonsense knowledge
 to predict the correct answers . It contains 12,102 questions with one correct answer and four distractor answers.
 The dataset is provided in two major training/validation/testing set splits: "Random split" which is the main evaluation
  split, and "Question token split", see paper for details.
"""

_URL = "https://s3.amazonaws.com/commensenseqa/"
_URLS = {
    "train": _URL + "train_rand_split.jsonl",
    "dev": _URL + "dev_rand_split.jsonl",
    "test": _URL + "test_rand_split_no_answers.jsonl",
}


class CommonsenseQa(datasets.GeneratorBasedBuilder):
    """TODO(commonsense_qa): Short description of my dataset."""

    # TODO(commonsense_qa): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # These are the features of your dataset like images, labels ...
        features = datasets.Features(
            {
                "answerKey": datasets.Value("string"),
                "question": datasets.Value("string"),
                "choices": datasets.features.Sequence(
                    {
                        "label": datasets.Value("string"),
                        "text": datasets.Value("string"),
                    }
                ),
            }
        )
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=features,
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://www.tau-datasets.org/commonsenseqa",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        download_urls = _URLS

        downloaded_files = dl_manager.download_and_extract(download_urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"], "split": "train"}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": downloaded_files["dev"],
                    "split": "dev",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": downloaded_files["test"],
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        # TODO(commonsense_qa): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                question = data["question"]
                choices = question["choices"]
                labels = [label["label"] for label in choices]
                texts = [text["text"] for text in choices]
                stem = question["stem"]
                if split == "test":
                    answerkey = ""
                else:
                    answerkey = data["answerKey"]

                yield id_, {
                    "answerKey": answerkey,
                    "question": stem,
                    "choices": {"label": labels, "text": texts},
                }



"""TODO(sciTail): Add a description here."""


import csv
import json
import os
import textwrap

import datasets


# TODO(sciTail): BibTeX citation
_CITATION = """\
inproceedings{scitail,
     Author = {Tushar Khot and Ashish Sabharwal and Peter Clark},
     Booktitle = {AAAI},
     Title = {{SciTail}: A Textual Entailment Dataset from Science Question Answering},
     Year = {2018}
}
"""

# TODO(sciTail):
_DESCRIPTION = """\
The SciTail dataset is an entailment dataset created from multiple-choice science exams and web sentences. Each question
and the correct answer choice are converted into an assertive statement to form the hypothesis. We use information
retrieval to obtain relevant text from a large text corpus of web sentences, and use these sentences as a premise P. We
crowdsource the annotation of such premise-hypothesis pair as supports (entails) or not (neutral), in order to create
the SciTail dataset. The dataset contains 27,026 examples with 10,101 examples with entails label and 16,925 examples
with neutral label
"""

_URL = "http://data.allenai.org.s3.amazonaws.com/downloads/SciTailV1.1.zip"


class ScitailConfig(datasets.BuilderConfig):

    """BuilderConfig for Xquad"""

    def __init__(self, **kwargs):
        """

        Args:
            **kwargs: keyword arguments forwarded to super.
        """
        super(ScitailConfig, self).__init__(version=datasets.Version("1.1.0", ""), **kwargs)


class Scitail(datasets.GeneratorBasedBuilder):
    """TODO(sciTail): Short description of my dataset."""

    # TODO(sciTail): Set up version.
    VERSION = datasets.Version("1.1.0")
    BUILDER_CONFIGS = [
        ScitailConfig(
            name="snli_format",
            description="JSONL format used by SNLI with a JSON object corresponding to each entailment example in each line.",
        ),
        ScitailConfig(
            name="tsv_format", description="Tab-separated format with three columns: premise hypothesis label"
        ),
        ScitailConfig(
            name="dgem_format",
            description="Tab-separated format used by the DGEM model: premise hypothesis label hypothesis graph structure",
        ),
        ScitailConfig(
            name="predictor_format",
            description=textwrap.dedent(
                """\
          AllenNLP predictors work only with JSONL format. This folder contains the SciTail train/dev/test in JSONL format
        so that it can be loaded into the predictors. Each line is a JSON object with the following keys:
        gold_label : the example label from {entails, neutral}
        sentence1: the premise
        sentence2: the hypothesis
        sentence2_structure: structure from the hypothesis """
            ),
        ),
    ]

    def _info(self):
        # TODO(sciTail): Specifies the datasets.DatasetInfo object
        if self.config.name == "snli_format":
            return datasets.DatasetInfo(
                # This is the description that will appear on the datasets page.
                description=_DESCRIPTION,
                # datasets.features.FeatureConnectors
                features=datasets.Features(
                    {
                        "sentence1_binary_parse": datasets.Value("string"),
                        "sentence1_parse": datasets.Value("string"),
                        "sentence1": datasets.Value("string"),
                        "sentence2_parse": datasets.Value("string"),
                        "sentence2": datasets.Value("string"),
                        "annotator_labels": datasets.features.Sequence(datasets.Value("string")),
                        "gold_label": datasets.Value("string")
                        # These are the features of your dataset like images, labels ...
                    }
                ),
                # If there's a common (input, target) tuple from the features,
                # specify them here. They'll be used if as_supervised=True in
                # builder.as_dataset.
                supervised_keys=None,
                # Homepage of the dataset for documentation
                homepage="https://allenai.org/data/scitail",
                citation=_CITATION,
            )
        elif self.config.name == "tsv_format":
            return datasets.DatasetInfo(
                # This is the description that will appear on the datasets page.
                description=_DESCRIPTION,
                # datasets.features.FeatureConnectors
                features=datasets.Features(
                    {
                        "premise": datasets.Value("string"),
                        "hypothesis": datasets.Value("string"),
                        "label": datasets.Value("string")
                        # These are the features of your dataset like images, labels ...
                    }
                ),
                # If there's a common (input, target) tuple from the features,
                # specify them here. They'll be used if as_supervised=True in
                # builder.as_dataset.
                supervised_keys=None,
                # Homepage of the dataset for documentation
                homepage="https://allenai.org/data/scitail",
                citation=_CITATION,
            )
        elif self.config.name == "predictor_format":
            return datasets.DatasetInfo(
                # This is the description that will appear on the datasets page.
                description=_DESCRIPTION,
                # datasets.features.FeatureConnectors
                features=datasets.Features(
                    {
                        "answer": datasets.Value("string"),
                        "sentence2_structure": datasets.Value("string"),
                        "sentence1": datasets.Value("string"),
                        "sentence2": datasets.Value("string"),
                        "gold_label": datasets.Value("string"),
                        "question": datasets.Value("string")
                        # These are the features of your dataset like images, labels ...
                    }
                ),
                # If there's a common (input, target) tuple from the features,
                # specify them here. They'll be used if as_supervised=True in
                # builder.as_dataset.
                supervised_keys=None,
                # Homepage of the dataset for documentation
                homepage="https://allenai.org/data/scitail",
                citation=_CITATION,
            )
        elif self.config.name == "dgem_format":
            return datasets.DatasetInfo(
                # This is the description that will appear on the datasets page.
                description=_DESCRIPTION,
                # datasets.features.FeatureConnectors
                features=datasets.Features(
                    {
                        "premise": datasets.Value("string"),
                        "hypothesis": datasets.Value("string"),
                        "label": datasets.Value("string"),
                        "hypothesis_graph_structure": datasets.Value("string")
                        # These are the features of your dataset like images, labels ...
                    }
                ),
                # If there's a common (input, target) tuple from the features,
                # specify them here. They'll be used if as_supervised=True in
                # builder.as_dataset.
                supervised_keys=None,
                # Homepage of the dataset for documentation
                homepage="https://allenai.org/data/scitail",
                citation=_CITATION,
            )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(sciTail): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        data_dir = os.path.join(dl_dir, "SciTailV1.1")
        snli = os.path.join(data_dir, "snli_format")
        dgem = os.path.join(data_dir, "dgem_format")
        tsv = os.path.join(data_dir, "tsv_format")
        predictor = os.path.join(data_dir, "predictor_format")
        if self.config.name == "snli_format":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(snli, "scitail_1.0_train.txt")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(snli, "scitail_1.0_test.txt")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(snli, "scitail_1.0_dev.txt")},
                ),
            ]
        elif self.config.name == "tsv_format":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(tsv, "scitail_1.0_train.tsv")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(tsv, "scitail_1.0_test.tsv")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(tsv, "scitail_1.0_dev.tsv")},
                ),
            ]
        elif self.config.name == "predictor_format":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(predictor, "scitail_1.0_structure_train.jsonl")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(predictor, "scitail_1.0_structure_test.jsonl")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(predictor, "scitail_1.0_structure_dev.jsonl")},
                ),
            ]
        elif self.config.name == "dgem_format":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(dgem, "scitail_1.0_structure_train.tsv")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(dgem, "scitail_1.0_structure_test.tsv")},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={"filepath": os.path.join(dgem, "scitail_1.0_structure_dev.tsv")},
                ),
            ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(sciTail): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            if self.config.name == "snli_format":
                for id_, row in enumerate(f):
                    data = json.loads(row)

                    yield id_, {
                        "sentence1_binary_parse": data["sentence1_binary_parse"],
                        "sentence1_parse": data["sentence1_parse"],
                        "sentence1": data["sentence1"],
                        "sentence2_parse": data["sentence2_parse"],
                        "sentence2": data["sentence2"],
                        "annotator_labels": data["annotator_labels"],
                        "gold_label": data["gold_label"],
                    }
            elif self.config.name == "tsv_format":
                data = csv.reader(f, delimiter="\t")
                for id_, row in enumerate(data):
                    yield id_, {"premise": row[0], "hypothesis": row[1], "label": row[2]}
            elif self.config.name == "dgem_format":
                data = csv.reader(f, delimiter="\t")
                for id_, row in enumerate(data):
                    yield id_, {
                        "premise": row[0],
                        "hypothesis": row[1],
                        "label": row[2],
                        "hypothesis_graph_structure": row[3],
                    }
            elif self.config.name == "predictor_format":
                for id_, row in enumerate(f):
                    data = json.loads(row)
                    yield id_, {
                        "answer": data["answer"],
                        "sentence2_structure": data["sentence2_structure"],
                        "sentence1": data["sentence1"],
                        "sentence2": data["sentence2"],
                        "gold_label": data["gold_label"],
                        "question": data["question"],
                    }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import datasets


_CITATION = """\
@InProceedings{WikiAtomicEdits,
  title = {{WikiAtomicEdits: A Multilingual Corpus of Wikipedia Edits for Modeling Language and Discourse}},
  author = {Faruqui, Manaal and Pavlick, Ellie and Tenney, Ian and Das, Dipanjan},
  booktitle = {Proc. of EMNLP},
  year = {2018}
}
"""

_DESCRIPTION = """\
A dataset of atomic wikipedia edits containing insertions and deletions of a contiguous chunk of text in a sentence. This dataset contains ~43 million edits across 8 languages.

An atomic edit is defined as an edit e applied to a natural language expression S as the insertion, deletion, or substitution of a sub-expression P such that both the original expression S and the resulting expression e(S) are well-formed semantic constituents (MacCartney, 2009). In this corpus, we release such atomic insertions and deletions made to sentences in wikipedia.
"""

_HOMEPAGE = "https://github.com/google-research-datasets/wiki-atomic-edits"
_VERSION = "1.0.0"

_URL = {
    "german": {
        "insertions": "https://storage.googleapis.com/wiki-atomic-edits/german/insertions.tsv.gz",
        "deletions": "https://storage.googleapis.com/wiki-atomic-edits/german/deletions.tsv.gz",
    },
    "english": {
        "insertions": "https://storage.googleapis.com/wiki-atomic-edits/english/insertions.tsv.gz",
        "deletions": "https://storage.googleapis.com/wiki-atomic-edits/english/deletions.tsv.gz",
    },
    "spanish": {
        "insertions": "https://storage.googleapis.com/wiki-atomic-edits/spanish/insertions.tsv.gz",
        "deletions": "https://storage.googleapis.com/wiki-atomic-edits/spanish/deletions.tsv.gz",
    },
    "french": {
        "insertions": "https://storage.googleapis.com/wiki-atomic-edits/french/insertions.tsv.gz",
        "deletions": "https://storage.googleapis.com/wiki-atomic-edits/french/deletions.tsv.gz",
    },
    "italian": {
        "insertions": "https://storage.googleapis.com/wiki-atomic-edits/italian/insertions.tsv.gz",
        "deletions": "https://storage.googleapis.com/wiki-atomic-edits/italian/deletions.tsv.gz",
    },
    "japanese": {
        "insertions": "https://storage.googleapis.com/wiki-atomic-edits/japanese/insertions.tsv.gz",
        "deletions": "https://storage.googleapis.com/wiki-atomic-edits/japanese/deletions.tsv.gz",
    },
    "russian": {
        "insertions": "https://storage.googleapis.com/wiki-atomic-edits/russian/insertions.tsv.gz",
        "deletions": "https://storage.googleapis.com/wiki-atomic-edits/russian/deletions.tsv.gz",
    },
    "chinese": {
        "insertions": "https://storage.googleapis.com/wiki-atomic-edits/chinese/insertions.tsv.gz",
        "deletions": "https://storage.googleapis.com/wiki-atomic-edits/chinese/deletions.tsv.gz",
    },
}

_LANG_EDITS = [
    ("german", "insertions"),
    ("german", "deletions"),
    ("english", "insertions"),
    ("english", "deletions"),
    ("spanish", "insertions"),
    ("spanish", "deletions"),
    ("french", "insertions"),
    ("french", "deletions"),
    ("italian", "insertions"),
    ("italian", "deletions"),
    ("japanese", "insertions"),
    ("japanese", "deletions"),
    ("russian", "insertions"),
    ("russian", "deletions"),
    ("chinese", "insertions"),
    ("chinese", "deletions"),
]


class WikiAtomicEditsConfig(datasets.BuilderConfig):
    def __init__(self, *args, language=None, edit_type=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.language = language
        self.edit_type = edit_type


class WikiAtomicEdits(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        WikiAtomicEditsConfig(
            name=f"{lang}_{edit_type}",
            language=lang,
            edit_type=edit_type,
            description=f"Language: {lang}, Edit type: {edit_type}",
            version=datasets.Version(_VERSION),
        )
        for lang, edit_type in _LANG_EDITS
    ]
    BUILDER_CONFIG_CLASS = WikiAtomicEditsConfig

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "base_sentence": datasets.Value("string"),
                    "phrase": datasets.Value("string"),
                    "edited_sentence": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        datapath = dl_manager.download_and_extract(_URL[self.config.language][self.config.edit_type])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": datapath},
            )
        ]

    def _generate_examples(self, datapath):
        with open(datapath, "r", encoding="utf-8") as data_file:
            next(data_file)
            for sentence_counter, sent in enumerate(data_file):
                sent = sent.split("\t")
                res = {
                    "id": sentence_counter,
                    "base_sentence": sent[0],
                    "phrase": sent[1],
                    "edited_sentence": sent[2],
                }
                yield sentence_counter, res



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Switchboard Dialog Act Corpus
The Switchboard Dialog Act Corpus (SwDA) extends the Switchboard-1 Telephone Speech Corpus, Release 2,
with turn/utterance-level dialog-act tags. The tags summarize syntactic, semantic, and pragmatic information
about the associated turn. The SwDA project was undertaken at UC Boulder in the late 1990s.

This script is a modified version of the original swda.py from https://github.com/cgpotts/swda/blob/master/swda.py from
the original corpus repo. Modifications are made to accommodate the HuggingFace Dataset project format.
"""


import csv
import datetime
import glob
import io
import os
import re

import datasets


# Citation as described here: https://github.com/cgpotts/swda#citation.
_CITATION = """\
@techreport{Jurafsky-etal:1997,
    Address = {Boulder, CO},
    Author = {Jurafsky, Daniel and Shriberg, Elizabeth and Biasca, Debra},
    Institution = {University of Colorado, Boulder Institute of Cognitive Science},
    Number = {97-02},
    Title = {Switchboard {SWBD}-{DAMSL} Shallow-Discourse-Function Annotation Coders Manual, Draft 13},
    Year = {1997}}

@article{Shriberg-etal:1998,
    Author = {Shriberg, Elizabeth and Bates, Rebecca and Taylor, Paul and Stolcke, Andreas and Jurafsky, Daniel and Ries, Klaus and Coccaro, Noah and Martin, Rachel and Meteer, Marie and Van Ess-Dykema, Carol},
    Journal = {Language and Speech},
    Number = {3--4},
    Pages = {439--487},
    Title = {Can Prosody Aid the Automatic Classification of Dialog Acts in Conversational Speech?},
    Volume = {41},
    Year = {1998}}

@article{Stolcke-etal:2000,
    Author = {Stolcke, Andreas and Ries, Klaus and Coccaro, Noah and Shriberg, Elizabeth and Bates, Rebecca and Jurafsky, Daniel and Taylor, Paul and Martin, Rachel and Meteer, Marie and Van Ess-Dykema, Carol},
    Journal = {Computational Linguistics},
    Number = {3},
    Pages = {339--371},
    Title = {Dialogue Act Modeling for Automatic Tagging and Recognition of Conversational Speech},
    Volume = {26},
    Year = {2000}}
"""

# Description of dataset gathered from: https://github.com/cgpotts/swda#overview.
_DESCRIPTION = """\
The Switchboard Dialog Act Corpus (SwDA) extends the Switchboard-1 Telephone Speech Corpus, Release 2 with
turn/utterance-level dialog-act tags. The tags summarize syntactic, semantic, and pragmatic information about the
associated turn. The SwDA project was undertaken at UC Boulder in the late 1990s.
The SwDA is not inherently linked to the Penn Treebank 3 parses of Switchboard, and it is far from straightforward to
align the two resources. In addition, the SwDA is not distributed with the Switchboard's tables of metadata about the
conversations and their participants.
"""

# Homepage gathered from: https://github.com/cgpotts/swda#overview.
_HOMEPAGE = "http://compprag.christopherpotts.net/swda.html"

# More details about the license: https://creativecommons.org/licenses/by-nc-sa/3.0/.
_LICENSE = "Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License"

# Dataset main url.
_URL = "https://github.com/cgpotts/swda/raw/master/swda.zip"

# Dialogue act tags - long version 217 dialogue acts labels.
_ACT_TAGS = [
    "b^m^r",
    "qw^r^t",
    "aa^h",
    "br^m",
    "fa^r",
    "aa,ar",
    "sd^e(^q)^r",
    "^2",
    "sd;qy^d",
    "oo",
    "bk^m",
    "aa^t",
    "cc^t",
    "qy^d^c",
    "qo^t",
    "ng^m",
    "qw^h",
    "qo^r",
    "aa",
    "qy^d^t",
    "qrr^d",
    "br^r",
    "fx",
    "sd,qy^g",
    "ny^e",
    "^h^t",
    "fc^m",
    "qw(^q)",
    "co",
    "o^t",
    "b^m^t",
    "qr^d",
    "qw^g",
    "ad(^q)",
    "qy(^q)",
    "na^r",
    "am^r",
    "qr^t",
    "ad^c",
    "qw^c",
    "bh^r",
    "h^t",
    "ft^m",
    "ba^r",
    "qw^d^t",
    "%",
    "t3",
    "nn",
    "bd",
    "h^m",
    "h^r",
    "sd^r",
    "qh^m",
    "^q^t",
    "sv^2",
    "ft",
    "ar^m",
    "qy^h",
    "sd^e^m",
    "qh^r",
    "cc",
    "fp^m",
    "ad",
    "qo",
    "na^m^t",
    "fo^c",
    "qy",
    "sv^e^r",
    "aap",
    "no",
    "aa^2",
    "sv(^q)",
    "sv^e",
    "nd",
    '"',
    "bf^2",
    "bk",
    "fp",
    "nn^r^t",
    "fa^c",
    "ny^t",
    "ny^c^r",
    "qw",
    "qy^t",
    "b",
    "fo",
    "qw^r",
    "am",
    "bf^t",
    "^2^t",
    "b^2",
    "x",
    "fc",
    "qr",
    "no^t",
    "bk^t",
    "bd^r",
    "bf",
    "^2^g",
    "qh^c",
    "ny^c",
    "sd^e^r",
    "br",
    "fe",
    "by",
    "^2^r",
    "fc^r",
    "b^m",
    "sd,sv",
    "fa^t",
    "sv^m",
    "qrr",
    "^h^r",
    "na",
    "fp^r",
    "o",
    "h,sd",
    "t1^t",
    "nn^r",
    "cc^r",
    "sv^c",
    "co^t",
    "qy^r",
    "sv^r",
    "qy^d^h",
    "sd",
    "nn^e",
    "ny^r",
    "b^t",
    "ba^m",
    "ar",
    "bf^r",
    "sv",
    "bh^m",
    "qy^g^t",
    "qo^d^c",
    "qo^d",
    "nd^t",
    "aa^r",
    "sd^2",
    "sv;sd",
    "qy^c^r",
    "qw^m",
    "qy^g^r",
    "no^r",
    "qh(^q)",
    "sd;sv",
    "bf(^q)",
    "+",
    "qy^2",
    "qw^d",
    "qy^g",
    "qh^g",
    "nn^t",
    "ad^r",
    "oo^t",
    "co^c",
    "ng",
    "^q",
    "qw^d^c",
    "qrr^t",
    "^h",
    "aap^r",
    "bc^r",
    "sd^m",
    "bk^r",
    "qy^g^c",
    "qr(^q)",
    "ng^t",
    "arp",
    "h",
    "bh",
    "sd^c",
    "^g",
    "o^r",
    "qy^c",
    "sd^e",
    "fw",
    "ar^r",
    "qy^m",
    "bc",
    "sv^t",
    "aap^m",
    "sd;no",
    "ng^r",
    "bf^g",
    "sd^e^t",
    "o^c",
    "b^r",
    "b^m^g",
    "ba",
    "t1",
    "qy^d(^q)",
    "nn^m",
    "ny",
    "ba,fe",
    "aa^m",
    "qh",
    "na^m",
    "oo(^q)",
    "qw^t",
    "na^t",
    "qh^h",
    "qy^d^m",
    "ny^m",
    "fa",
    "qy^d",
    "fc^t",
    "sd(^q)",
    "qy^d^r",
    "bf^m",
    "sd(^q)^t",
    "ft^t",
    "^q^r",
    "sd^t",
    "sd(^q)^r",
    "ad^t",
]

# Damsl dialogue act tags version - short version 43 dialogue acts labels.
_DAMSL_ACT_TAGS = [
    "ad",
    "qo",
    "qy",
    "arp_nd",
    "sd",
    "h",
    "bh",
    "no",
    "^2",
    "^g",
    "ar",
    "aa",
    "sv",
    "bk",
    "fp",
    "qw",
    "b",
    "ba",
    "t1",
    "oo_co_cc",
    "+",
    "ny",
    "qw^d",
    "x",
    "qh",
    "fc",
    'fo_o_fw_"_by_bc',
    "aap_am",
    "%",
    "bf",
    "t3",
    "nn",
    "bd",
    "ng",
    "^q",
    "br",
    "qy^d",
    "fa",
    "^h",
    "b^m",
    "ft",
    "qrr",
    "na",
]


class Swda(datasets.GeneratorBasedBuilder):
    """
    This is the HuggingFace Dataset class for swda.

    Switchboard Dialog Act Corpus Hugging Face Dataset class.
    The Switchboard Dialog Act Corpus (SwDA) extends the Switchboard-1 Telephone Speech Corpus, Release 2,
    with turn/utterance-level dialog-act tags. The tags summarize syntactic, semantic, and pragmatic information
    about the associated turn. The SwDA project was undertaken at UC Boulder in the late 1990s.

    """

    # Urls for each split train-dev-test.
    _URLS = {
        "train": "https://github.com/NathanDuran/Probabilistic-RNN-DA-Classifier/raw/master/data/train_split.txt",
        "dev": "https://github.com/NathanDuran/Probabilistic-RNN-DA-Classifier/raw/master/data/dev_split.txt",
        "test": "https://github.com/NathanDuran/Probabilistic-RNN-DA-Classifier/raw/master/data/test_split.txt",
    }

    def _info(self):
        """
        Specify the datasets.DatasetInfo object which contains information and typings for the dataset.
        """

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types.
            features=datasets.Features(
                {
                    "swda_filename": datasets.Value("string"),
                    "ptb_basename": datasets.Value("string"),
                    "conversation_no": datasets.Value("int64"),
                    "transcript_index": datasets.Value("int64"),
                    "act_tag": datasets.ClassLabel(num_classes=217, names=_ACT_TAGS),
                    "damsl_act_tag": datasets.ClassLabel(num_classes=43, names=_DAMSL_ACT_TAGS),
                    "caller": datasets.Value("string"),
                    "utterance_index": datasets.Value("int64"),
                    "subutterance_index": datasets.Value("int64"),
                    "text": datasets.Value("string"),
                    "pos": datasets.Value("string"),
                    "trees": datasets.Value("string"),
                    "ptb_treenumbers": datasets.Value("string"),
                    "talk_day": datasets.Value("string"),
                    "length": datasets.Value("int64"),
                    "topic_description": datasets.Value("string"),
                    "prompt": datasets.Value("string"),
                    "from_caller": datasets.Value("int64"),
                    "from_caller_sex": datasets.Value("string"),
                    "from_caller_education": datasets.Value("int64"),
                    "from_caller_birth_year": datasets.Value("int64"),
                    "from_caller_dialect_area": datasets.Value("string"),
                    "to_caller": datasets.Value("int64"),
                    "to_caller_sex": datasets.Value("string"),
                    "to_caller_education": datasets.Value("int64"),
                    "to_caller_birth_year": datasets.Value("int64"),
                    "to_caller_dialect_area": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """
        Returns SplitGenerators.
        This method is tasked with downloading/extracting the data and defining the splits.

         Args:
            dl_manager (:obj:`datasets.utils.download_manager.DownloadManager`):
                Download manager to download and extract data files from urls.

        Returns:
            :obj:`list[str]`:
                List of paths to data.
        """

        # Download extract and return path of data file.
        dl_dir = dl_manager.download_and_extract(_URL)
        # Use swda/ folder.
        data_dir = os.path.join(dl_dir, "swda")
        # Handle partitions files.
        urls_to_download = self._URLS
        # Download extract and return paths of split files.
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            # Return whole data path and train splits file downloaded path.
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"data_dir": data_dir, "split_file": downloaded_files["train"]}
            ),
            # Return whole data path and dev splits file downloaded path.
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"data_dir": data_dir, "split_file": downloaded_files["dev"]},
            ),
            # Return whole data path and train splits file downloaded path.
            datasets.SplitGenerator(
                name=datasets.Split.TEST, gen_kwargs={"data_dir": data_dir, "split_file": downloaded_files["test"]}
            ),
        ]

    def _generate_examples(self, data_dir, split_file):
        """
        Yields examples.
        This method will receive as arguments the `gen_kwargs` defined in the previous `_split_generators` method.
        It is in charge of opening the given file and yielding (key, example) tuples from the dataset
        The key is not important, it's more here for legacy reason (legacy from tfds).

        Args:
            data_dir (:obj:`str`):
                Path where is downloaded dataset.

            split_file (:obj:`str`):
                Path of split file used for train-dev-test.

        Returns:
            :obj:`list[str]`:
                List of paths to data.
        """

        # Read in the split file.
        split_file = io.open(file=split_file, mode="r", encoding="utf-8").read().splitlines()
        # Read in corpus data using split files.
        corpus = CorpusReader(src_dirname=data_dir, split_file=split_file)
        # Generate examples.
        for i_trans, trans in enumerate(corpus.iter_transcripts()):
            for i_utt, utt in enumerate(trans.utterances):
                id_ = str(i_trans) + ":" + str(i_utt)
                yield id_, {feature: utt[feature] for feature in self.info.features.keys()}


class CorpusReader:
    """Class for reading in the corpus and iterating through its values."""

    def __init__(self, src_dirname, split_file=None):
        """
        Reads in the data from `src_dirname` (should be the root of the
        corpus).  Assumes that the metadata file `swda-metadata.csv` is
        in the main directory of the corpus, using that file to build
        the `Metadata` object used throughout.

        Args:
            src_dirname (:obj:`str`):
                Path where swda folder with all data.

            split_file (:obj:`list[str`, `optional`):
                List of file names used in a split (train, dev or test). This argument is optional and it will have a None value attributed inside the function.

        """

        self.src_dirname = src_dirname
        metadata_filename = os.path.join(src_dirname, "swda-metadata.csv")
        self.metadata = Metadata(metadata_filename)
        self.split_file = split_file

    def iter_transcripts(
        self,
    ):
        """
        Iterate through the transcripts.

        Returns:
            :obj:`Transcript`:
                Transcript instance.
        """

        # All files names.
        filenames = glob.glob(os.path.join(self.src_dirname, "sw*", "*.csv"))
        # If no split files are mentioned just use all files.
        self.split_file = filenames if self.split_file is None else self.split_file
        # Filter out desired file names
        filenames = [
            file for file in filenames if os.path.basename(file).split("_")[-1].split(".")[0] in self.split_file
        ]
        for filename in sorted(filenames):
            # Yield the Transcript instance:
            yield Transcript(filename, self.metadata)

    def iter_utterances(
        self,
    ):
        """
        Iterate through the utterances.

        Returns:
            :obj:`Transcript.utterances`:
                Utterance instance object.
        """

        for trans in self.iter_transcripts():
            for utt in trans.utterances:
                # Yield the Utterance instance:
                yield utt


class Metadata:
    """
    Basically an internal method for organizing the tables of metadata
    from the original Switchboard transcripts and linking them with
    the dialog acts.
    """

    def __init__(self, metadata_filename):
        """
        Turns the CSV file into a dictionary mapping Switchboard
        conversation_no integers values to dictionaries of values. All
        the keys correspond to the column names in the original
        tables.

        Args:
            metadata_filename (:obj:`str`):
                The CSV file swda-metadata.csv (should be in the main
                folder of the swda directory).

        """
        self.metadata_filename = metadata_filename
        self.metadata = {}
        self.get_metadata()

    def get_metadata(self):
        """
        Build the dictionary self.metadata mapping conversation_no to
        dictionaries of values (str, int, or datatime, as
        appropriate).
        """

        csvreader = csv.reader(open(self.metadata_filename))
        header = next(csvreader)
        for row in csvreader:
            d = dict(list(zip(header, row)))
            # Add integer number features to metadata.
            for key in (
                "conversation_no",
                "length",
                "from_caller",
                "to_caller",
                "from_caller_education",
                "to_caller_education",
            ):
                d[key] = int(d[key])
            talk_day = d["talk_day"]
            talk_year = int("19" + talk_day[:2])
            talk_month = int(talk_day[2:4])
            talk_day = int(talk_day[4:])
            # Make sure to convert date time to string to match PyArrow data formats.
            d["talk_day"] = datetime.datetime(year=talk_year, month=talk_month, day=talk_day).strftime("%m/%d/%Y")
            d["from_caller_birth_year"] = int(d["from_caller_birth_year"])
            d["to_caller_birth_year"] = int(d["to_caller_birth_year"])
            self.metadata[d["conversation_no"]] = d

    def __getitem__(self, val):
        """
        Val should be a key in self.metadata; returns the
        corresponding value.

        Args:
            val (:obj:`str`):
                Key in self.metadata.

        Returns:
            :obj::
                Corresponding value.

        """

        return self.metadata[val]


class Utterance:
    """
    The central object of interest. The attributes correspond to the
    values of the class variable header:

    """

    # Metadata header file.
    header = [
        "swda_filename",
        "ptb_basename",
        "conversation_no",
        "transcript_index",
        "act_tag",
        "caller",
        "utterance_index",
        "subutterance_index",
        "text",
        "pos",
        "trees",
        "ptb_treenumbers",
    ]

    def __init__(self, row, transcript_metadata):
        """
        Args:
            row (:obj:`list`):
                A row from one of the corpus CSV files.

            transcript_metadata (:obj:`dict`):
                A Metadata value based on the current `conversation_no`.

        """

        # Utterance data:
        for i in range(len(Utterance.header)):
            att_name = Utterance.header[i]
            row_value = None
            if i < len(row):
                row_value = row[i].strip()
            # Special handling of non-string values.
            if att_name == "trees":
                if row_value:
                    # Origianl code returned list of nltk.tree and used `[Tree.fromstring(t) for t in row_value.split("|||")]`.
                    # Since we're returning str we don't need to make any mondifications to row_value.
                    row_value = row_value
                else:
                    row_value = ""  # []
            elif att_name == "ptb_treenumbers":
                if row_value:
                    row_value = row_value  # list(map(int, row_value.split("|||")))
                else:
                    row_value = ""  # []
            elif att_name == "act_tag":
                # I thought these conjoined tags were meant to be split.
                # The docs suggest that they are single tags, thought,
                # so skip this conditional and let it be treated as a str.
                # row_value = re.split(r"\s*[,;]\s*", row_value)
                # `` Transcription errors (typos, obvious mistranscriptions) are
                # marked with a "*" after the discourse tag.''
                # These are removed for this version.
                row_value = row_value.replace("*", "")
            elif att_name in ("conversation_no", "transcript_index", "utterance_index", "subutterance_index"):
                row_value = int(row_value)
            # Add attribute.
            setattr(self, att_name, row_value)
        # Make sure conversation number matches.
        assert self.conversation_no == transcript_metadata["conversation_no"]
        # Add rest of missing metadata
        [setattr(self, key, value) for key, value in transcript_metadata.items()]
        # Add damsl tags.
        setattr(self, "damsl_act_tag", self.damsl_act_tag())

    def __getitem__(self, feature):
        """
        Return utterance features as dictionary. It allows us to call an utterance object as a dictionary.
        It contains same keys as attributes.

        Args:
            feature (:obj:`str`):
                Feature value of utterance that is part of attributes.

        Returns:
            :obj:
                Value of feature from utterance. Value type can vary.
        """

        return vars(self)[feature]

    def damsl_act_tag(
        self,
    ):
        """
        Seeks to duplicate the tag simplification described at the
        Coders' Manual: http://www.stanford.edu/~jurafsky/ws97/manual.august1.html
        """

        d_tags = []
        tags = re.split(r"\s*[,;]\s*", self.act_tag)
        for tag in tags:
            if tag in ("qy^d", "qw^d", "b^m"):
                pass
            elif tag == "nn^e":
                tag = "ng"
            elif tag == "ny^e":
                tag = "na"
            else:
                tag = re.sub(r"(.)\^.*", r"\1", tag)
                tag = re.sub(r"[\(\)@*]", "", tag)
                if tag in ("qr", "qy"):
                    tag = "qy"
                elif tag in ("fe", "ba"):
                    tag = "ba"
                elif tag in ("oo", "co", "cc"):
                    tag = "oo_co_cc"
                elif tag in ("fx", "sv"):
                    tag = "sv"
                elif tag in ("aap", "am"):
                    tag = "aap_am"
                elif tag in ("arp", "nd"):
                    tag = "arp_nd"
                elif tag in ("fo", "o", "fw", '"', "by", "bc"):
                    tag = 'fo_o_fw_"_by_bc'
            d_tags.append(tag)
        # Dan J says (p.c.) that it makes sense to take the first;
        # there are only a handful of examples with 2 tags here.
        return d_tags[0]


class Transcript:
    """
    Transcript instances are basically just containers for lists of
    utterances and transcript-level metadata, accessible via
    attributes.
    """

    def __init__(self, swda_filename, metadata):
        """
        Sets up all the attribute values:

        Args:
            swda_filename (:obj:`str`):
                The filename for this transcript.

            metadata (:obj:`str` or `Metadata`):
                If a string, then assumed to be the metadata filename, and
                the metadata is created from that filename. If a `Metadata`
                object, then used as the needed metadata directly.

        """

        self.swda_filename = swda_filename
        # If the supplied value is a filename:
        if isinstance(metadata, str) or isinstance(metadata, str):
            self.metadata = Metadata(metadata)
        else:  # Where the supplied value is already a Metadata object.
            self.metadata = metadata
        # Get the file rows:
        rows = list(csv.reader(open(self.swda_filename, "rt")))
        # Ge the header and remove it from the rows:
        self.header = rows[0]
        rows.pop(0)
        # Extract the conversation_no to get the meta-data. Use the
        # header for this in case the column ordering is ever changed:
        row0dict = dict(list(zip(self.header, rows[1])))
        self.conversation_no = int(row0dict["conversation_no"])
        # The ptd filename in the right format for the current OS:
        self.ptd_basename = os.sep.join(row0dict["ptb_basename"].split("/"))
        # The dictionary of metadata for this transcript:
        transcript_metadata = self.metadata[self.conversation_no]
        for key, val in transcript_metadata.items():
            setattr(self, key, transcript_metadata[key])
        # Create the utterance list:
        self.utterances = [Utterance(x, transcript_metadata) for x in rows]
        # Coder's Manual: ``We also removed any line with a "@"
        # (since @ marked slash-units with bad segmentation).''
        self.utterances = [u for u in self.utterances if not re.search(r"[@]", u.act_tag)]



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""KdConv: Chinese multi-domain Knowledge-driven Conversionsation dataset"""


import json
import os

import datasets


_CITATION = """\
@inproceedings{zhou-etal-2020-kdconv,
    title = "{K}d{C}onv: A {C}hinese Multi-domain Dialogue Dataset Towards Multi-turn Knowledge-driven Conversation",
    author = "Zhou, Hao  and
      Zheng, Chujie  and
      Huang, Kaili  and
      Huang, Minlie  and
      Zhu, Xiaoyan",
    booktitle = "Proceedings of the 58th Annual Meeting of the Association for Computational Linguistics",
    month = jul,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/2020.acl-main.635",
    doi = "10.18653/v1/2020.acl-main.635",
    pages = "7098--7108",
}
"""


_DESCRIPTION = """\
KdConv is a Chinese multi-domain Knowledge-driven Conversionsation dataset, grounding the topics in multi-turn \
conversations to knowledge graphs. KdConv contains 4.5K conversations from three domains (film, music, and travel), \
and 86K utterances with an average turn number of 19.0. These conversations contain in-depth discussions on related \
topics and natural transition between multiple topics, while the corpus can also used for exploration of transfer \
learning and domain adaptation.\
"""


_HOMEPAGE = "https://github.com/thu-coai/KdConv"


_LICENSE = "Apache License 2.0"


_URL = "https://github.com/thu-coai/KdConv/archive/master.zip"

_DOMAINS = ["travel", "music", "film"]
_DATA_TYPES = ["dialogues", "knowledge_base"]


class KdConv(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.1.0")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name=domain + "_" + type,
            description="This part of dataset covers {0} domain and {1} data " "of the corpus".format(domain, type),
        )
        for domain in _DOMAINS
        for type in _DATA_TYPES
    ] + [
        datasets.BuilderConfig(
            name="all_" + type,
            description="This part of dataset covers all domains and {0} data of " "the corpus".format(type),
        )
        for type in _DATA_TYPES
    ]

    DEFAULT_CONFIG_NAME = "all_dialogues"

    def _info(self):
        if "dialogues" in self.config.name:
            features = datasets.Features(
                {
                    "messages": datasets.Sequence(
                        {
                            "message": datasets.Value("string"),
                            "attrs": datasets.Sequence(
                                {
                                    "attrname": datasets.Value("string"),
                                    "attrvalue": datasets.Value("string"),
                                    "name": datasets.Value("string"),
                                }
                            ),
                        }
                    ),
                    "name": datasets.Value("string"),
                    "domain": datasets.Value("string"),
                }
            )
        else:
            features = datasets.Features(
                {
                    "head_entity": datasets.Value("string"),
                    "kb_triplets": datasets.Sequence(datasets.Sequence(datasets.Value("string"))),
                    "domain": datasets.Value("string"),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        data_dir = dl_manager.download_and_extract(_URL)
        base_dir = os.path.join(os.path.join(data_dir, "KdConv-master"), "data")
        if "dialogues" in self.config.name:
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "data_dir": base_dir,
                        "split": "train",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={"data_dir": base_dir, "split": "test"},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={
                        "data_dir": base_dir,
                        "split": "dev",
                    },
                ),
            ]
        else:
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "data_dir": base_dir,
                        "split": "train",
                    },
                ),
            ]

    def _generate_examples(self, data_dir, split):
        """Yields examples."""
        if "dialogues" in self.config.name:
            if "all" in self.config.name:
                file_dict = {
                    domain: os.path.join(os.path.join(data_dir, domain), split + ".json") for domain in _DOMAINS
                }
            else:
                domain = self.config.name.split("_")[0]
                file_dict = {domain: os.path.join(os.path.join(data_dir, domain), split + ".json")}
            id_ = -1
            for domain, filepath in file_dict.items():
                with open(filepath, encoding="utf-8") as f:
                    conversations = json.load(f)
                    for conversation in conversations:
                        id_ += 1
                        conversation["domain"] = domain
                        for turn in conversation["messages"]:
                            if "attrs" in turn:
                                attrnames = [kb_triplet.get("attrname", "") for kb_triplet in turn["attrs"]]
                                attrvalues = [kb_triplet.get("attrvalue", "") for kb_triplet in turn["attrs"]]
                                names = [kb_triplet.get("name", "") for kb_triplet in turn["attrs"]]
                            else:
                                attrnames, attrvalues, names = [], [], []
                            turn["attrs"] = {"attrname": attrnames, "attrvalue": attrvalues, "name": names}

                        yield id_, conversation
        else:
            if "all" in self.config.name:
                file_dict = {
                    domain: os.path.join(os.path.join(data_dir, domain), "kb_" + domain + ".json")
                    for domain in _DOMAINS
                }
            else:
                domain = self.config.name.split("_")[0]
                file_dict = {domain: os.path.join(os.path.join(data_dir, domain), "kb_" + domain + ".json")}

            id_ = -1
            for domain, filepath in file_dict.items():
                with open(filepath, encoding="utf-8") as f:
                    kb_dict = json.load(f)
                    for head_entity, kb_triplets in kb_dict.items():
                        id_ += 1
                        yield id_, {"head_entity": head_entity, "kb_triplets": kb_triplets, "domain": domain}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The LAMBADA dataset."""


import glob
import os
import tarfile

import datasets


_CITATION = """\
@InProceedings{paperno-EtAl:2016:P16-1,
  author    = {Paperno, Denis  and  Kruszewski, Germ\'{a}n  and  Lazaridou,
Angeliki  and  Pham, Ngoc Quan  and  Bernardi, Raffaella  and  Pezzelle,
Sandro  and  Baroni, Marco  and  Boleda, Gemma  and  Fernandez, Raquel},
  title     = {The {LAMBADA} dataset: Word prediction requiring a broad
discourse context},
  booktitle = {Proceedings of the 54th Annual Meeting of the Association for
Computational Linguistics (Volume 1: Long Papers)},
  month     = {August},
  year      = {2016},
  address   = {Berlin, Germany},
  publisher = {Association for Computational Linguistics},
  pages     = {1525--1534},
  url       = {http://www.aclweb.org/anthology/P16-1144}
}
"""

_DESCRIPTION = """
The LAMBADA evaluates the capabilities of computational models
for text understanding by means of a word prediction task.
LAMBADA is a collection of narrative passages sharing the characteristic
that human subjects are able to guess their last word if
they are exposed to the whole passage, but not if they
only see the last sentence preceding the target word.
To succeed on LAMBADA, computational models cannot
simply rely on local context, but must be able to
keep track of information in the broader discourse.

The LAMBADA dataset is extracted from BookCorpus and
consists of 10'022 passages, divided into 4'869 development
and 5'153 test passages. The training data for language
models to be tested on LAMBADA include the full text
of 2'662 novels (disjoint from those in dev+test),
comprising 203 million words.
"""

_URL = "https://zenodo.org/record/2630551/files/lambada-dataset.tar.gz"


class Lambada(datasets.GeneratorBasedBuilder):
    """LAMBADA dataset."""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="plain_text",
            description="Plain text",
            version=VERSION,
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "domain": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage="https://zenodo.org/record/2630551#.X8UP76pKiIa",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URL)

        # Extracting (un-taring) the training data
        tar_file = tarfile.open(os.path.join(data_dir, "train-novels.tar"))
        tar_file.extractall(path=data_dir)
        tar_file.close()

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "train-novels"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, "lambada_test_plain_text.txt"), "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "lambada_development_plain_text.txt"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        if split == "train":
            recursion_pattern = f"{filepath}/*/*.txt"
            for idx, novel_path in enumerate(glob.iglob(recursion_pattern, recursive=True)):
                domain = os.path.dirname(novel_path).split("/")[-1]
                with open(novel_path, encoding="utf-8") as novel:
                    text = novel.read()
                    yield idx, {"text": text.strip(), "domain": domain}
        else:
            with open(filepath, encoding="utf-8") as f:
                data = f.read().splitlines()
                for idx, text in enumerate(data):
                    yield idx, {"text": text, "domain": None}



import os
import re
import shutil
from argparse import ArgumentParser, Namespace

from datasets.commands import BaseDatasetsCLICommand
from datasets.utils.logging import get_logger


HIGHLIGHT_MESSAGE_PRE = """<<<<<<< This should probably be modified because it mentions: """

HIGHLIGHT_MESSAGE_POST = """=======
>>>>>>>
"""

TO_HIGHLIGHT = [
    "TextEncoderConfig",
    "ByteTextEncoder",
    "SubwordTextEncoder",
    "encoder_config",
    "maybe_build_from_corpus",
    "manual_dir",
]

TO_CONVERT = [
    # (pattern, replacement)
    # Order is important here for some replacements
    (r"tfds\.core", r"datasets"),
    (r"tf\.io\.gfile\.GFile", r"open"),
    (r"tf\.([\w\d]+)", r"datasets.Value('\1')"),
    (r"tfds\.features\.Text\(\)", r"datasets.Value('string')"),
    (r"tfds\.features\.Text\(", r"datasets.Value('string'),"),
    (r"features\s*=\s*tfds.features.FeaturesDict\(", r"features=datasets.Features("),
    (r"tfds\.features\.FeaturesDict\(", r"dict("),
    (r"The TensorFlow Datasets Authors", r"The TensorFlow Datasets Authors and the HuggingFace Datasets Authors"),
    (r"tfds\.", r"datasets."),
    (r"dl_manager\.manual_dir", r"self.config.data_dir"),
    (r"self\.builder_config", r"self.config"),
]


def convert_command_factory(args: Namespace):
    """
    Factory function used to convert a model TF 1.0 checkpoint in a PyTorch checkpoint.

    Returns: ConvertCommand
    """
    return ConvertCommand(args.tfds_path, args.datasets_directory)


class ConvertCommand(BaseDatasetsCLICommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        """
        Register this command to argparse so it's available for the datasets-cli

        Args:
            parser: Root parser to register command-specific arguments
        """
        train_parser = parser.add_parser(
            "convert",
            help="Convert a TensorFlow Datasets dataset to a HuggingFace Datasets dataset.",
        )
        train_parser.add_argument(
            "--tfds_path",
            type=str,
            required=True,
            help="Path to a TensorFlow Datasets folder to convert or a single tfds file to convert.",
        )
        train_parser.add_argument(
            "--datasets_directory", type=str, required=True, help="Path to the HuggingFace Datasets folder."
        )
        train_parser.set_defaults(func=convert_command_factory)

    def __init__(self, tfds_path: str, datasets_directory: str, *args):
        self._logger = get_logger("datasets-cli/converting")

        self._tfds_path = tfds_path
        self._datasets_directory = datasets_directory

    def run(self):
        if os.path.isdir(self._tfds_path):
            abs_tfds_path = os.path.abspath(self._tfds_path)
        elif os.path.isfile(self._tfds_path):
            abs_tfds_path = os.path.dirname(self._tfds_path)
        else:
            raise ValueError("--tfds_path is neither a directory nor a file. Please check path.")

        abs_datasets_path = os.path.abspath(self._datasets_directory)

        self._logger.info(f"Converting datasets from {abs_tfds_path} to {abs_datasets_path}")

        utils_files = []
        with_manual_update = []
        imports_to_builder_map = {}

        if os.path.isdir(self._tfds_path):
            file_names = os.listdir(abs_tfds_path)
        else:
            file_names = [os.path.basename(self._tfds_path)]

        for f_name in file_names:
            self._logger.info(f"Looking at file {f_name}")
            input_file = os.path.join(abs_tfds_path, f_name)
            output_file = os.path.join(abs_datasets_path, f_name)

            if not os.path.isfile(input_file) or "__init__" in f_name or "_test" in f_name or ".py" not in f_name:
                self._logger.info("Skipping file")
                continue

            with open(input_file, "r", encoding="utf-8") as f:
                lines = f.readlines()

            out_lines = []
            is_builder = False
            needs_manual_update = False
            tfds_imports = []
            for line in lines:
                out_line = line

                # Convert imports
                if "import tensorflow.compat.v2 as tf" in out_line:
                    continue
                elif "@tfds.core" in out_line:
                    continue
                elif "builder=self" in out_line:
                    continue
                elif "import tensorflow_datasets.public_api as tfds" in out_line:
                    out_line = "import datasets\n"
                elif "import tensorflow" in out_line:
                    # order is important here
                    out_line = ""
                    continue
                elif "from absl import logging" in out_line:
                    out_line = "from datasets import logging\n"
                elif "getLogger" in out_line:
                    out_line = out_line.replace("getLogger", "get_logger")
                elif any(expression in out_line for expression in TO_HIGHLIGHT):
                    needs_manual_update = True
                    to_remove = list(filter(lambda e: e in out_line, TO_HIGHLIGHT))
                    out_lines.append(HIGHLIGHT_MESSAGE_PRE + str(to_remove) + "\n")
                    out_lines.append(out_line)
                    out_lines.append(HIGHLIGHT_MESSAGE_POST)
                    continue
                else:
                    for pattern, replacement in TO_CONVERT:
                        out_line = re.sub(pattern, replacement, out_line)

                # Take care of saving utilities (to later move them together with main script)
                if "tensorflow_datasets" in out_line:
                    match = re.match(r"from\stensorflow_datasets.*import\s([^\.\r\n]+)", out_line)
                    tfds_imports.extend(imp.strip() for imp in match.group(1).split(","))
                    out_line = "from . import " + match.group(1)

                # Check we have not forget anything
                if "tf." in out_line or "tfds." in out_line or "tensorflow_datasets" in out_line:
                    raise ValueError(f"Error converting {out_line.strip()}")

                if "GeneratorBasedBuilder" in out_line or "BeamBasedBuilder" in out_line:
                    is_builder = True
                out_lines.append(out_line)

            if is_builder or "wmt" in f_name:
                # We create a new directory for each dataset
                dir_name = f_name.replace(".py", "")
                output_dir = os.path.join(abs_datasets_path, dir_name)
                output_file = os.path.join(output_dir, f_name)
                os.makedirs(output_dir, exist_ok=True)
                self._logger.info(f"Adding directory {output_dir}")
                imports_to_builder_map.update({imp: output_dir for imp in tfds_imports})
            else:
                # Utilities will be moved at the end
                utils_files.append(output_file)

            if needs_manual_update:
                with_manual_update.append(output_file)

            with open(output_file, "w", encoding="utf-8") as f:
                f.writelines(out_lines)
            self._logger.info(f"Converted in {output_file}")

        for utils_file in utils_files:
            try:
                f_name = os.path.basename(utils_file)
                dest_folder = imports_to_builder_map[f_name.replace(".py", "")]
                self._logger.info(f"Moving {dest_folder} to {utils_file}")
                shutil.copy(utils_file, dest_folder)
            except KeyError:
                self._logger.error(f"Cannot find destination folder for {utils_file}. Please copy manually.")

        if with_manual_update:
            for file_path in with_manual_update:
                self._logger.warning(
                    f"You need to manually update file {file_path} to remove configurations using 'TextEncoderConfig'."
                )



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""BillSum Dataset."""


import json
import os

import datasets


_CITATION = """
@misc{kornilova2019billsum,
    title={BillSum: A Corpus for Automatic Summarization of US Legislation},
    author={Anastassia Kornilova and Vlad Eidelman},
    year={2019},
    eprint={1910.00523},
    archivePrefix={arXiv},
    primaryClass={cs.CL}
}
"""

_DESCRIPTION = """
BillSum, summarization of US Congressional and California state bills.

There are several features:
  - text: bill text.
  - summary: summary of the bills.
  - title: title of the bills.
features for us bills. ca bills does not have.
  - text_len: number of chars in text.
  - sum_len: number of chars in summary.
"""

_URL = "https://drive.google.com/uc?export=download&id=1g89WgFHMRbr4QrvA0ngh26PY081Nv3lx"

_DOCUMENT = "text"
_SUMMARY = "summary"


class Billsum(datasets.GeneratorBasedBuilder):
    """BillSum Dataset."""

    # 2.0.0 data source updated to filter near duplicates.
    # 3.0.0  none of the test examples are 'near duplicates' of an example in the
    #   train set AND they dont have the same title, regardless of similarity.
    VERSION = datasets.Version("3.0.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    _DOCUMENT: datasets.Value("string"),
                    _SUMMARY: datasets.Value("string"),
                    "title": datasets.Value("string"),
                }
            ),
            supervised_keys=(_DOCUMENT, _SUMMARY),
            homepage="https://github.com/FiscalNote/BillSum",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        dl_path = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"path": os.path.join(dl_path, "us_train_data_final_OFFICIAL.jsonl"), "key": "bill_id"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"path": os.path.join(dl_path, "us_test_data_final_OFFICIAL.jsonl"), "key": "bill_id"},
            ),
            datasets.SplitGenerator(
                name="ca_test",
                gen_kwargs={"path": os.path.join(dl_path, "ca_test_data_final_OFFICIAL.jsonl"), "key": "external_id"},
            ),
        ]

    def _generate_examples(self, path=None, key=None):
        """Yields examples."""
        with open(path, encoding="utf-8") as f:
            for line in f:
                # in us bills, json has fields:
                #   text, summary, title, bill_id, text_len, sum_len
                # in ca bills, json has fields:
                #   text, summary, title, external_id
                d = json.loads(line)
                yield d[key], {k: d[k] for k in [_DOCUMENT, _SUMMARY, "title"]}



"""TODO(coarse_discourse): Add a description here."""


import json
import os

import datasets


# TODO(coarse_discourse): BibTeX citation
_CITATION = """\
@inproceedings{coarsediscourse, title={Characterizing Online Discussion Using Coarse Discourse Sequences}, author={Zhang, Amy X. and Culbertson, Bryan and Paritosh, Praveen}, booktitle={Proceedings of the 11th International AAAI Conference on Weblogs and Social Media}, series={ICWSM '17}, year={2017}, location = {Montreal, Canada} }
"""

# TODO(coarse_discourse):
_DESCRIPTION = """\
dataset contains discourse annotation and relation on threads from reddit during 2016
"""
_URL = "https://github.com/google-research-datasets/coarse-discourse/archive/master.zip"


class CoarseDiscourse(datasets.GeneratorBasedBuilder):
    """TODO(coarse_discourse): Short description of my dataset."""

    # TODO(coarse_discourse): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(coarse_discourse): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    # These are the features of your dataset like images, labels ...
                    "title": datasets.Value("string"),
                    "is_self_post": datasets.Value("bool"),
                    "subreddit": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "majority_link": datasets.Value("string"),
                    "is_first_post": datasets.Value("bool"),
                    "majority_type": datasets.Value("string"),
                    "id_post": datasets.Value("string"),
                    "post_depth": datasets.Value("int32"),
                    "in_reply_to": datasets.Value("string"),
                    "annotations": datasets.features.Sequence(
                        {
                            "annotator": datasets.Value("string"),
                            "link_to_post": datasets.Value("string"),
                            "main_type": datasets.Value("string"),
                        }
                    ),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/google-research-datasets/coarse-discourse",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(coarse_discourse): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(dl_dir, "coarse-discourse-master", "coarse_discourse_dataset.json")
                },
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(coarse_discourse): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                url = data.get("url", "")
                is_self_post = data.get("is_self_post", "")
                subreddit = data.get("subreddit", "")
                title = data.get("title", "")
                posts = data.get("posts", "")
                for id1, post in enumerate(posts):
                    maj_link = post.get("majority_link", "")
                    maj_type = post.get("majority_type", "")
                    id_post = post.get("id", "")
                    is_first_post = post.get("is_firs_post", "")
                    post_depth = post.get("post_depth", -1)
                    in_reply_to = post.get("in_reply_to", "")
                    annotations = post["annotations"]
                    annotators = [annotation.get("annotator", "") for annotation in annotations]
                    main_types = [annotation.get("main_type", "") for annotation in annotations]
                    link_posts = [annotation.get("linkk_to_post", "") for annotation in annotations]

                    yield str(id_) + "_" + str(id1), {
                        "title": title,
                        "is_self_post": is_self_post,
                        "subreddit": subreddit,
                        "url": url,
                        "majority_link": maj_link,
                        "is_first_post": is_first_post,
                        "majority_type": maj_type,
                        "id_post": id_post,
                        "post_depth": post_depth,
                        "in_reply_to": in_reply_to,
                        "annotations": {"annotator": annotators, "link_to_post": link_posts, "main_type": main_types},
                    }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""ParaCrawl (Bitextor) parallel open-source machine translation benchmark."""


import collections

import datasets


_DESCRIPTION = "Web-Scale Parallel Corpora for Official European Languages."

_BENCHMARK_URL = "https://paracrawl.eu/releases.html"

_CITATION = """\
@misc {paracrawl,
    title  = {ParaCrawl},
    year   = {2018},
    url    = {http://paracrawl.eu/download.html.}
}
"""

_BASE_DATA_URL_FORMAT_STR = (
    "https://s3.amazonaws.com/web-language-models/" "paracrawl/release4/en-{target_lang}.bicleaner07." "txt.gz"
)


def _target_languages():
    """Create the sorted dictionary of language codes, and language names.

    Returns:
      The sorted dictionary as an instance of `collections.OrderedDict`.
    """
    langs = {
        "bg": "Bulgarian",
        "cs": "Czech",
        "da": "Danish",
        "de": "German",
        "el": "Greek",
        "es": "Spanish",
        "et": "Estonian",
        "fi": "Finnish",
        "fr": "French",
        "ga": "Irish",
        "hr": "Croatian",
        "hu": "Hungarian",
        "it": "Italian",
        "lt": "Lithuanian",
        "lv": "Latvian",
        "mt": "Maltese",
        "nl": "Dutch",
        "pl": "Polish",
        "pt": "Portuguese",
        "ro": "Romanian",
        "sk": "Slovak",
        "sl": "Slovenian",
        "sv": "Swedish",
    }
    return collections.OrderedDict(sorted(langs.items()))


class ParaCrawlConfig(datasets.BuilderConfig):
    """BuilderConfig for ParaCrawl."""

    def __init__(self, target_language=None, **kwargs):
        """BuilderConfig for ParaCrawl.

        Args:
            for the `datasets.features.text.TextEncoder` used for the features feature.
          target_language: Target language that will be used to translate to from
            English which is always the source language. It has to contain 2-letter
            coded strings. For example: "se", "hu".
          **kwargs: Keyword arguments forwarded to super.
        """
        # Validate the target language.
        if target_language not in _target_languages():
            raise ValueError("Invalid target language: %s " % target_language)

        # Initialize the base class.
        name = "en%s" % (target_language)

        description = ("Translation dataset from English to %s.") % (target_language)
        super(ParaCrawlConfig, self).__init__(name=name, description=description, **kwargs)

        # Store the attributes.

        self.target_language = target_language
        self.data_url = _BASE_DATA_URL_FORMAT_STR.format(target_lang=target_language)


class ParaCrawl(datasets.GeneratorBasedBuilder):
    """ParaCrawl machine translation dataset."""

    # Version history:
    # 1.0.0: S3 (new shuffling, sharding and slicing mechanism).
    # 0.1.0: Initial version.
    BUILDER_CONFIGS = [
        # The version below does not refer to the version of the released
        # database. It only indicates the version of the TFDS integration.
        ParaCrawlConfig(  # pylint: disable=g-complex-comprehension
            target_language=target_language,
            version=datasets.Version("1.0.0"),
        )
        for target_language in _target_languages()
    ]

    def _info(self):
        target_language = self.config.target_language
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {"translation": datasets.features.Translation(languages=("en", target_language))}
            ),
            supervised_keys=("en", target_language),
            homepage=_BENCHMARK_URL,
            citation=_CITATION,
        )

    def _vocab_text_gen(self, files, language):
        for _, ex in self._generate_examples(**files):
            yield ex[language]

    def _split_generators(self, dl_manager):
        # Download the data file.
        data_file = dl_manager.download_and_extract({"data_file": self.config.data_url})

        # Return the single split of the data.
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs=data_file)]

    def _generate_examples(self, data_file):
        """This function returns the examples in the raw (text) form."""
        target_language = self.config.target_language

        with open(data_file, encoding="utf-8") as f:
            for idx, line in enumerate(f):
                line_parts = line.strip().split("\t")
                if len(line_parts) != 2:
                    msg = (
                        "Wrong data format in line {}. The line '{}' does " "not have exactly one delimiter."
                    ).format(idx, line)
                    raise ValueError(msg)
                source, target = line_parts[0].strip(), line_parts[1].strip()
                yield idx, {"translation": {"en": source, target_language: target}}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""QUAC (Question Answering in Context)."""


import json

import datasets


_CITATION = """\
@inproceedings{choi-etal-2018-quac,
title = "QUAC: Question answering in context",
abstract = "We present QuAC, a dataset for Question Answering in Context that contains 14K information-seeking QA dialogs (100K questions in total). The dialogs involve two crowd workers: (1) a student who poses a sequence of freeform questions to learn as much as possible about a hidden Wikipedia text, and (2) a teacher who answers the questions by providing short excerpts from the text. QuAC introduces challenges not found in existing machine comprehension datasets: its questions are often more open-ended, unanswerable, or only meaningful within the dialog context, as we show in a detailed qualitative evaluation. We also report results for a number of reference models, including a recently state-of-the-art reading comprehension architecture extended to model dialog context. Our best model underperforms humans by 20 F1, suggesting that there is significant room for future work on this data. Dataset, baseline, and leaderboard available at http://quac.ai.",
author = "Eunsol Choi and He He and Mohit Iyyer and Mark Yatskar and Yih, {Wen Tau} and Yejin Choi and Percy Liang and Luke Zettlemoyer",
year = "2018",
language = "English (US)",
series = "Proceedings of the 2018 Conference on Empirical Methods in Natural Language Processing, EMNLP 2018",
publisher = "Association for Computational Linguistics",
pages = "2174--2184",
editor = "Ellen Riloff and David Chiang and Julia Hockenmaier and Jun'ichi Tsujii",
booktitle = "Proceedings of the 2018 Conference on Empirical Methods in Natural Language Processing, EMNLP 2018",
note = "2018 Conference on Empirical Methods in Natural Language Processing, EMNLP 2018 ; Conference date: 31-10-2018 Through 04-11-2018",
}
"""

_DESCRIPTION = """\
Question Answering in Context is a dataset for modeling, understanding,
and participating in information seeking dialog. Data instances consist
of an interactive dialog between two crowd workers: (1) a student who
poses a sequence of freeform questions to learn as much as possible
about a hidden Wikipedia text, and (2) a teacher who answers the questions
by providing short excerpts (spans) from the text. QuAC introduces
challenges not found in existing machine comprehension datasets: its
questions are often more open-ended, unanswerable, or only meaningful
within the dialog context.
"""

_HOMEPAGE = "https://quac.ai/"

_LICENSE = "MIT"

_URLs = {
    "train": "https://s3.amazonaws.com/my89public/quac/train_v0.2.json",
    "validation": "https://s3.amazonaws.com/my89public/quac/val_v0.2.json",
}


class Quac(datasets.GeneratorBasedBuilder):
    """QuAC (Question Answering in Context)."""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="plain_text",
            description="Plain text",
            version=VERSION,
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "dialogue_id": datasets.Value("string"),
                    "wikipedia_page_title": datasets.Value("string"),
                    "background": datasets.Value("string"),
                    "section_title": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "turn_ids": datasets.Sequence(datasets.Value("string")),
                    "questions": datasets.Sequence(datasets.Value("string")),
                    "followups": datasets.Sequence(datasets.ClassLabel(names=["y", "n", "m"])),
                    "yesnos": datasets.Sequence(datasets.ClassLabel(names=["y", "n", "x"])),
                    "answers": datasets.Sequence(
                        {
                            "texts": datasets.Sequence(datasets.Value("string")),
                            "answer_starts": datasets.Sequence(datasets.Value("int32")),
                        }
                    ),
                    "orig_answers": {
                        "texts": datasets.Sequence(datasets.Value("string")),
                        "answer_starts": datasets.Sequence(datasets.Value("int32")),
                    },
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URLs)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["validation"],
                },
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            squad = json.load(f)
            for section in squad["data"]:
                wiki_page_title = section.get("title", "").strip()
                background = section.get("background", "").strip()
                section_title = section.get("section_title", "").strip()

                for dialogue in section["paragraphs"]:
                    context = dialogue["context"].strip()
                    dialogue_id = dialogue["id"]

                    followups = []
                    yesnos = []
                    questions = []
                    turn_ids = []
                    answers = []
                    orig_answers = {"texts": [], "answer_starts": []}

                    for turn in dialogue["qas"]:
                        followups.append(turn["followup"])
                        yesnos.append(turn["yesno"])
                        questions.append(turn["question"])
                        turn_ids.append(turn["id"])

                        ans_ = {
                            "texts": [t["text"].strip() for t in turn["answers"]],
                            "answer_starts": [t["answer_start"] for t in turn["answers"]],
                        }
                        answers.append(ans_)

                        orig_answers["texts"].append(turn["orig_answer"]["text"])
                        orig_answers["answer_starts"].append(turn["orig_answer"]["answer_start"])

                    yield dialogue_id, {
                        "dialogue_id": dialogue_id,
                        "wikipedia_page_title": wiki_page_title,
                        "background": background,
                        "section_title": section_title,
                        "context": context,
                        "turn_ids": turn_ids,
                        "questions": questions,
                        "followups": followups,
                        "yesnos": yesnos,
                        "answers": answers,
                        "orig_answers": orig_answers,
                    }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Hotel Reviews in Arabic language"""


import os

import datasets
from datasets.tasks import TextClassification


_DESCRIPTION = """\
This dataset contains 93700 hotel reviews in Arabic language.\
The hotel reviews were collected from Booking.com website during June/July 2016.\
The reviews are expressed in Modern Standard Arabic as well as dialectal Arabic.\
The following table summarize some tatistics on the HARD Dataset.
"""

_CITATION = """\
@incollection{elnagar2018hotel,
  title={Hotel Arabic-reviews dataset construction for sentiment analysis applications},
  author={Elnagar, Ashraf and Khalifa, Yasmin S and Einea, Anas},
  booktitle={Intelligent Natural Language Processing: Trends and Applications},
  pages={35--52},
  year={2018},
  publisher={Springer}
}
"""

_DOWNLOAD_URL = "https://raw.githubusercontent.com/elnagara/HARD-Arabic-Dataset/master/data/balanced-reviews.zip"


class HardConfig(datasets.BuilderConfig):
    """BuilderConfig for Hard."""

    def __init__(self, **kwargs):
        """BuilderConfig for Hard.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(HardConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)


class Hard(datasets.GeneratorBasedBuilder):
    """Hard dataset."""

    BUILDER_CONFIGS = [
        HardConfig(
            name="plain_text",
            description="Plain text",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(
                        names=[
                            "1",
                            "2",
                            "3",
                            "4",
                            "5",
                        ]
                    ),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/elnagara/HARD-Arabic-Dataset",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="text", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        data_dir = dl_manager.download_and_extract(_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"directory": os.path.join(data_dir, "balanced-reviews.txt")}
            ),
        ]

    def _generate_examples(self, directory):
        """Generate examples."""
        with open(directory, mode="r", encoding="utf-16") as file:
            for id_, line in enumerate(file.read().splitlines()[1:]):
                _, _, rating, _, _, _, review_text = line.split("\t")
                yield str(id_), {"text": review_text, "label": rating}



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""NEWSROOM Dataset."""


import json
import os

import datasets


_CITATION = """
@inproceedings{N18-1065,
  author    = {Grusky, Max and Naaman, Mor and Artzi, Yoav},
  title     = {NEWSROOM: A Dataset of 1.3 Million Summaries
               with Diverse Extractive Strategies},
  booktitle = {Proceedings of the 2018 Conference of the
               North American Chapter of the Association for
               Computational Linguistics: Human Language Technologies},
  year      = {2018},
}
"""

_DESCRIPTION = """
NEWSROOM is a large dataset for training and evaluating summarization systems.
It contains 1.3 million articles and summaries written by authors and
editors in the newsrooms of 38 major publications.

Dataset features includes:
  - text: Input news text.
  - summary: Summary for the news.
And additional features:
  - title: news title.
  - url: url of the news.
  - date: date of the article.
  - density: extractive density.
  - coverage: extractive coverage.
  - compression: compression ratio.
  - density_bin: low, medium, high.
  - coverage_bin: extractive, abstractive.
  - compression_bin: low, medium, high.

This dataset can be downloaded upon requests. Unzip all the contents
"train.jsonl, dev.josnl, test.jsonl" to the tfds folder.

"""

_DOCUMENT = "text"
_SUMMARY = "summary"
_ADDITIONAL_TEXT_FEATURES = [
    "title",
    "url",
    "date",
    "density_bin",
    "coverage_bin",
    "compression_bin",
]
_ADDITIONAL_FLOAT_FEATURES = [
    "density",
    "coverage",
    "compression",
]


class Newsroom(datasets.GeneratorBasedBuilder):
    """NEWSROOM Dataset."""

    VERSION = datasets.Version("1.0.0")

    @property
    def manual_download_instructions(self):
        return """\
  You should download the dataset from http://lil.datasets.cornell.edu/newsroom/
  The webpage requires registration.
  To unzip the .tar file run `tar -zxvf complete.tar`. To unzip the .gz files
  run `gunzip train.json.gz` , ...
  After downloading, please put the files under the following names
  dev.jsonl, test.jsonl and train.jsonl in a dir of your choice,
  which will be used as a manual_dir, e.g. `~/.manual_dirs/newsroom`
  Newsroom can then be loaded via:
  `datasets.load_dataset("newsroom", data_dir="~/.manual_dirs/newsroom")`.
  """

    def _info(self):
        features = {k: datasets.Value("string") for k in [_DOCUMENT, _SUMMARY] + _ADDITIONAL_TEXT_FEATURES}
        features.update({k: datasets.Value("float32") for k in _ADDITIONAL_FLOAT_FEATURES})
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(features),
            supervised_keys=(_DOCUMENT, _SUMMARY),
            homepage="http://lil.datasets.cornell.edu/newsroom/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))
        if not os.path.exists(data_dir):
            raise FileNotFoundError(
                f"{data_dir} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('newsroom', data_dir=...)` that includes files unzipped from the reclor zip. Manual download instructions: {self.manual_download_instructions}"
            )
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"input_file": os.path.join(data_dir, "train.jsonl")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"input_file": os.path.join(data_dir, "dev.jsonl")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"input_file": os.path.join(data_dir, "test.jsonl")},
            ),
        ]

    def _generate_examples(self, input_file=None):
        """Yields examples."""
        with open(input_file, encoding="utf-8") as f:
            for i, line in enumerate(f):
                d = json.loads(line)
                # fields are "url", "archive", "title", "date", "text",
                #  "compression_bin", "density_bin", "summary", "density",
                #  "compression', "coverage", "coverage_bin",
                yield i, {
                    k: d[k] for k in [_DOCUMENT, _SUMMARY] + _ADDITIONAL_TEXT_FEATURES + _ADDITIONAL_FLOAT_FEATURES
                }
